
from typing import List, Union
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap({})
class ChangeValue(BaseModel):
    """ChangeValue - Typed representation of newValue/oldValue in diff changes.

    Live API responses return these as objects with xpath and value fields
    rather than plain strings.

    :param xpath: XPath expression identifying the changed element, defaults to None
    :type xpath: str, optional
    :param value: The actual value content, defaults to None
    :type value: str, optional
    """

    def __init__(self, xpath: str = SENTINEL, value: str = SENTINEL, **kwargs):
        if xpath is not SENTINEL:
            self.xpath = xpath
        if value is not SENTINEL:
            self.value = value
        self._kwargs = kwargs


@JsonMap({})
class KeyPart(BaseModel):
    """KeyPart

    :param attribute: Attribute name, defaults to None
    :type attribute: str, optional
    :param value: Attribute value, defaults to None
    :type value: str, optional
    """

    def __init__(self, attribute: str = SENTINEL, value: str = SENTINEL, **kwargs):
        """KeyPart

        :param attribute: Attribute name, defaults to None
        :type attribute: str, optional
        :param value: Attribute value, defaults to None
        :type value: str, optional
        """
        if attribute is not SENTINEL:
            self.attribute = attribute
        if value is not SENTINEL:
            self.value = value
        self._kwargs = kwargs


@JsonMap({"element_name": "elementName", "key_part": "key-part"})
class ChangeElementKey1(BaseModel):
    """ChangeElementKey1

    :param element_name: Name of the element, defaults to None
    :type element_name: str, optional
    :param key_part: key_part, defaults to None
    :type key_part: KeyPart, optional
    """

    def __init__(
        self, element_name: str = SENTINEL, key_part: KeyPart = SENTINEL, **kwargs
    ):
        """ChangeElementKey1

        :param element_name: Name of the element, defaults to None
        :type element_name: str, optional
        :param key_part: key_part (may be object, list, or empty list in live responses), defaults to None
        :type key_part: KeyPart, optional
        """
        if element_name is not SENTINEL:
            self.element_name = element_name
        if key_part is not SENTINEL:
            # Normalize key-part: live API returns object, one-item list, or empty list
            if isinstance(key_part, list):
                self._key_parts = self._define_list(key_part, KeyPart) or []
                self.key_part = self._key_parts[0] if self._key_parts else None
            else:
                self.key_part = self._define_object(key_part, KeyPart)
                self._key_parts = [self.key_part] if self.key_part else []
        self._kwargs = kwargs

    @property
    def key_parts(self):
        """All key-part entries as a list."""
        return getattr(self, '_key_parts', [])


@JsonMap(
    {
        "type_": "type",
        "changed_particle_name": "changedParticleName",
        "element_key": "elementKey",
        "new_value": "newValue",
    }
)
class AdditionChange(BaseModel):
    """AdditionChange

    :param type_: Type of change (e.g., element), defaults to None
    :type type_: str, optional
    :param changed_particle_name: Name of the particle that changed, defaults to None
    :type changed_particle_name: str, optional
    :param element_key: element_key, defaults to None
    :type element_key: ChangeElementKey1, optional
    :param new_value: New value of the element in the diff, defaults to None
    :type new_value: Union[str, ChangeValue], optional
    """

    def __init__(
        self,
        type_: str = SENTINEL,
        changed_particle_name: str = SENTINEL,
        element_key: ChangeElementKey1 = SENTINEL,
        new_value: str = SENTINEL,
        **kwargs
    ):
        if type_ is not SENTINEL:
            self.type_ = type_
        if changed_particle_name is not SENTINEL:
            self.changed_particle_name = changed_particle_name
        if element_key is not SENTINEL:
            self.element_key = self._define_object(element_key, ChangeElementKey1)
        if new_value is not SENTINEL:
            if isinstance(new_value, dict):
                self.new_value = self._define_object(new_value, ChangeValue)
            else:
                self.new_value = new_value
        self._kwargs = kwargs


@JsonMap({})
class Addition(BaseModel):
    """Addition

    :param total: Total number of additions, defaults to None
    :type total: int, optional
    :param change: change, defaults to None
    :type change: List[AdditionChange], optional
    """

    def __init__(
        self, total: int = SENTINEL, change: List[AdditionChange] = SENTINEL, **kwargs
    ):
        if total is not SENTINEL:
            self.total = total
        if change is not SENTINEL:
            # Normalize singleton dict to list
            if isinstance(change, dict):
                change = [change]
            self.change = self._define_list(change, AdditionChange)
        self._kwargs = kwargs


@JsonMap({"element_name": "elementName"})
class ChangeElementKey2(BaseModel):
    """ChangeElementKey2

    :param element_name: Name of the element, defaults to None
    :type element_name: str, optional
    """

    def __init__(self, element_name: str = SENTINEL, **kwargs):
        if element_name is not SENTINEL:
            self.element_name = element_name
        self._kwargs = kwargs


@JsonMap(
    {
        "type_": "type",
        "changed_particle_name": "changedParticleName",
        "element_key": "elementKey",
        "new_value": "newValue",
        "old_value": "oldValue",
    }
)
class ModificationChange(BaseModel):
    """ModificationChange

    :param type_: Type of modification (e.g., attribute), defaults to None
    :type type_: str, optional
    :param changed_particle_name: Name of the particle that was modified, defaults to None
    :type changed_particle_name: str, optional
    :param element_key: element_key, defaults to None
    :type element_key: ChangeElementKey2, optional
    :param new_value: New value of the attribute, defaults to None
    :type new_value: Union[str, ChangeValue], optional
    :param old_value: Old value of the attribute, defaults to None
    :type old_value: Union[str, ChangeValue], optional
    """

    def __init__(
        self,
        type_: str = SENTINEL,
        changed_particle_name: str = SENTINEL,
        element_key: ChangeElementKey2 = SENTINEL,
        new_value: str = SENTINEL,
        old_value: str = SENTINEL,
        **kwargs
    ):
        if type_ is not SENTINEL:
            self.type_ = type_
        if changed_particle_name is not SENTINEL:
            self.changed_particle_name = changed_particle_name
        if element_key is not SENTINEL:
            self.element_key = self._define_object(element_key, ChangeElementKey2)
        if new_value is not SENTINEL:
            if isinstance(new_value, dict):
                self.new_value = self._define_object(new_value, ChangeValue)
            else:
                self.new_value = new_value
        if old_value is not SENTINEL:
            if isinstance(old_value, dict):
                self.old_value = self._define_object(old_value, ChangeValue)
            else:
                self.old_value = old_value
        self._kwargs = kwargs


@JsonMap({})
class Modification(BaseModel):
    """Modification

    :param total: Total number of modifications, defaults to None
    :type total: int, optional
    :param change: change, defaults to None
    :type change: List[ModificationChange], optional
    """

    def __init__(
        self, total: int = SENTINEL, change: List[ModificationChange] = SENTINEL, **kwargs
    ):
        if total is not SENTINEL:
            self.total = total
        if change is not SENTINEL:
            # Normalize singleton dict to list
            if isinstance(change, dict):
                change = [change]
            self.change = self._define_list(change, ModificationChange)
        self._kwargs = kwargs


@JsonMap(
    {
        "type_": "type",
        "changed_particle_name": "changedParticleName",
        "element_key": "elementKey",
        "old_value": "oldValue",
    }
)
class DeletionChange(BaseModel):
    """DeletionChange

    :param type_: Type of change (e.g., element), defaults to None
    :type type_: str, optional
    :param changed_particle_name: Name of the particle that changed, defaults to None
    :type changed_particle_name: str, optional
    :param element_key: element_key, defaults to None
    :type element_key: ChangeElementKey1, optional
    :param old_value: Old value of the element in the diff, defaults to None
    :type old_value: Union[str, ChangeValue], optional
    """

    def __init__(
        self,
        type_: str = SENTINEL,
        changed_particle_name: str = SENTINEL,
        element_key: ChangeElementKey1 = SENTINEL,
        old_value: str = SENTINEL,
        **kwargs
    ):
        if type_ is not SENTINEL:
            self.type_ = type_
        if changed_particle_name is not SENTINEL:
            self.changed_particle_name = changed_particle_name
        if element_key is not SENTINEL:
            self.element_key = self._define_object(element_key, ChangeElementKey1)
        if old_value is not SENTINEL:
            if isinstance(old_value, dict):
                self.old_value = self._define_object(old_value, ChangeValue)
            else:
                self.old_value = old_value
        self._kwargs = kwargs


@JsonMap({})
class Deletion(BaseModel):
    """Deletion

    :param total: Total number of deletions, defaults to None
    :type total: int, optional
    :param change: change, defaults to None
    :type change: List[DeletionChange], optional
    """

    def __init__(
        self, total: int = SENTINEL, change: List[DeletionChange] = SENTINEL, **kwargs
    ):
        if total is not SENTINEL:
            self.total = total
        if change is not SENTINEL:
            # Normalize singleton dict to list
            if isinstance(change, dict):
                change = [change]
            self.change = self._define_list(change, DeletionChange)
        self._kwargs = kwargs


@JsonMap({})
class GenericDiff(BaseModel):
    """GenericDiff

    :param addition: addition, defaults to None
    :type addition: Addition, optional
    :param deletion: deletion, defaults to None
    :type deletion: Deletion, optional
    :param modification: modification, defaults to None
    :type modification: Modification, optional
    """

    def __init__(
        self,
        addition: Addition = SENTINEL,
        deletion: Deletion = SENTINEL,
        modification: Modification = SENTINEL,
        **kwargs
    ):
        if addition is not SENTINEL:
            self.addition = self._define_object(addition, Addition)
        if deletion is not SENTINEL:
            self.deletion = self._define_object(deletion, Deletion)
        if modification is not SENTINEL:
            self.modification = self._define_object(modification, Modification)
        self._kwargs = kwargs


@JsonMap({"generic_diff": "GenericDiff"})
class ComponentDiffResponse(BaseModel):
    """ComponentDiffResponse

    :param message: Message providing details about the diffed components, defaults to None
    :type message: str, optional
    :param generic_diff: generic_diff, defaults to None
    :type generic_diff: GenericDiff, optional
    """

    def __init__(
        self, message: str = SENTINEL, generic_diff: GenericDiff = SENTINEL, **kwargs
    ):
        if message is not SENTINEL:
            self.message = message
        if generic_diff is not SENTINEL:
            self.generic_diff = self._define_object(generic_diff, GenericDiff)
        self._kwargs = kwargs


@JsonMap({"component_diff_response": "ComponentDiffResponse"})
class ComponentDiffResponseCreate(BaseModel):
    """ComponentDiffResponseCreate

    Wrapper for component diff responses. Handles both wrapped payloads
    (with a top-level ComponentDiffResponse key) and unwrapped payloads
    (where message/GenericDiff appear at the top level).

    :param component_diff_response: component_diff_response, defaults to None
    :type component_diff_response: ComponentDiffResponse, optional
    """

    def __init__(
        self, component_diff_response: ComponentDiffResponse = SENTINEL, **kwargs
    ):
        if component_diff_response is not SENTINEL:
            self.component_diff_response = self._define_object(
                component_diff_response, ComponentDiffResponse
            )
        elif kwargs.get('@type') == 'ComponentDiffResponse' or 'GenericDiff' in kwargs or 'message' in kwargs:
            # Unwrapped response: API returned diff data at top level without
            # the ComponentDiffResponse wrapper key. Build the inner response
            # from kwargs so public fields are populated.
            diff_data = {k: v for k, v in kwargs.items() if k != '@type'}
            self.component_diff_response = ComponentDiffResponse._unmap(diff_data)
            kwargs = {k: v for k, v in kwargs.items() if k not in ('message', 'GenericDiff', '@type')}
        self._kwargs = kwargs

    @property
    def message(self):
        """Proxy to component_diff_response.message for convenience."""
        cdr = getattr(self, 'component_diff_response', None)
        return getattr(cdr, 'message', None) if cdr else None

    @property
    def generic_diff(self):
        """Proxy to component_diff_response.generic_diff for convenience."""
        cdr = getattr(self, 'component_diff_response', None)
        return getattr(cdr, 'generic_diff', None) if cdr else None




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .queue_attributes import QueueAttributes


@JsonMap(
    {
        "atom_id": "AtomId",
        "destination_queue": "DestinationQueue",
        "source_queue": "SourceQueue",
    }
)
class MoveQueueRequest(BaseModel):
    """MoveQueueRequest

    :param atom_id: \(Required\) The Runtime ID that the queue or topic exists under and where you can find it in Runtime Management. You can find the Runtime ID for Runtimes, Runtime clusters, and Runtime clouds in the user interface by navigating to **Manage** \\> **Runtime Management** and viewing the **Runtime Information** panel for a selected container.
    :type atom_id: str
    :param destination_queue: \(Required\) An instance of a generalized queue object indicating the queue from which to send or obtain the messages. Contains information describing the desired queue.
    :type destination_queue: QueueAttributes
    :param source_queue: \(Required\) An instance of a generalized queue object indicating the queue from which to send or obtain the messages. Contains information describing the desired queue.
    :type source_queue: QueueAttributes
    """

    def __init__(
        self,
        atom_id: str,
        destination_queue: QueueAttributes,
        source_queue: QueueAttributes,
        **kwargs,
    ):
        """MoveQueueRequest

        :param atom_id: \(Required\) The Runtime ID that the queue or topic exists under and where you can find it in Runtime Management. You can find the Runtime ID for Runtimes, Runtime clusters, and Runtime clouds in the user interface by navigating to **Manage** \\> **Runtime Management** and viewing the **Runtime Information** panel for a selected container.
        :type atom_id: str
        :param destination_queue: \(Required\) An instance of a generalized queue object indicating the queue from which to send or obtain the messages. Contains information describing the desired queue.
        :type destination_queue: QueueAttributes
        :param source_queue: \(Required\) An instance of a generalized queue object indicating the queue from which to send or obtain the messages. Contains information describing the desired queue.
        :type source_queue: QueueAttributes
        """
        self.atom_id = atom_id
        self.destination_queue = self._define_object(destination_queue, QueueAttributes)
        self.source_queue = self._define_object(source_queue, QueueAttributes)
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap({"component_id": "componentId", "priority_branch": "priorityBranch"})
class ComponentMergeConfig(BaseModel):
    """ComponentMergeConfig

    :param component_id: The ID of the component to merge.
    :type component_id: str
    :param priority_branch: The priority branch for the merge., defaults to None
    :type priority_branch: str, optional
    """

    def __init__(
        self,
        component_id: str,
        priority_branch: str = SENTINEL,
        **kwargs,
    ):
        """ComponentMergeConfig

        :param component_id: The ID of the component to merge.
        :type component_id: str
        :param priority_branch: The priority branch for the merge., defaults to None
        :type priority_branch: str, optional
        """
        self.component_id = component_id
        if priority_branch is not SENTINEL:
            self.priority_branch = priority_branch
        self._kwargs = kwargs




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..execution_artifacts import ExecutionArtifactsService
from ...models import ExecutionArtifacts


class ExecutionArtifactsServiceAsync(ExecutionArtifactsService):
    """
    Async Wrapper for ExecutionArtifactsServiceAsync
    """

    def create_execution_artifacts(
        self, request_body: ExecutionArtifacts = None
    ) -> Awaitable[Union[ExecutionArtifacts, str]]:
        return to_async(super().create_execution_artifacts)(request_body)

    def download_execution_artifacts(
        self,
        request_body: ExecutionArtifacts = None,
        max_retries: int = 10,
        initial_delay: float = 2.0,
    ) -> Awaitable[bytes]:
        return to_async(super().download_execution_artifacts)(
            request_body, max_retries, initial_delay
        )




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .property import Property


@JsonMap({})
class CustomRuntimeProperties(BaseModel):
    """CustomRuntimeProperties

    :param property: property
    :type property: List[Property]
    """

    def __init__(self, property: List[Property], **kwargs):
        """CustomRuntimeProperties

        :param property: property
        :type property: List[Property]
        """
        self.property = self._define_list(property, Property)
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .branch_expression import BranchExpression, BranchExpressionGuard
from .branch_simple_expression import BranchSimpleExpression
from .branch_grouping_expression import BranchGroupingExpression


@JsonMap({})
class BranchQueryConfigQueryFilter(BaseModel):
    """BranchQueryConfigQueryFilter

    :param expression: expression
    :type expression: BranchExpression
    """

    def __init__(self, expression: BranchExpression, **kwargs):
        """BranchQueryConfigQueryFilter

        :param expression: expression
        :type expression: BranchExpression
        """
        self.expression = BranchExpressionGuard.return_one_of(expression)
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class BranchQueryConfig(BaseModel):
    """BranchQueryConfig

    :param query_filter: query_filter
    :type query_filter: BranchQueryConfigQueryFilter
    """

    def __init__(self, query_filter: BranchQueryConfigQueryFilter, **kwargs):
        """BranchQueryConfig

        :param query_filter: query_filter
        :type query_filter: BranchQueryConfigQueryFilter
        """
        self.query_filter = self._define_object(
            query_filter, BranchQueryConfigQueryFilter
        )
        self._kwargs = kwargs




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .tradacoms_connector_record_simple_expression import (
    TradacomsConnectorRecordSimpleExpression,
)
from .tradacoms_connector_record_grouping_expression import (
    TradacomsConnectorRecordGroupingExpression,
)


class TradacomsConnectorRecordExpressionGuard(OneOfBaseModel):
    class_list = {
        "TradacomsConnectorRecordSimpleExpression": TradacomsConnectorRecordSimpleExpression,
        "TradacomsConnectorRecordGroupingExpression": TradacomsConnectorRecordGroupingExpression,
    }


TradacomsConnectorRecordExpression = Union[
    TradacomsConnectorRecordSimpleExpression, TradacomsConnectorRecordGroupingExpression
]




from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


class ChangeType(Enum):
    """An enumeration representing different categories.

    :cvar ADDED: "ADDED"
    :vartype ADDED: str
    :cvar MODIFIED: "MODIFIED"
    :vartype MODIFIED: str
    :cvar DELETED: "DELETED"
    :vartype DELETED: str
    """

    ADDED = "ADDED"
    MODIFIED = "MODIFIED"
    DELETED = "DELETED"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, ChangeType._member_map_.values()))


class Resolution(Enum):
    """An enumeration representing different categories.

    :cvar OVERRIDE: "OVERRIDE"
    :vartype OVERRIDE: str
    :cvar KEEPDESTINATION: "KEEP_DESTINATION"
    :vartype KEEPDESTINATION: str
    """

    OVERRIDE = "OVERRIDE"
    KEEPDESTINATION = "KEEP_DESTINATION"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, Resolution._member_map_.values()))


class MergeRequestDetailStage(Enum):
    """An enumeration representing different categories.

    :cvar DRAFTED: "DRAFTED"
    :vartype DRAFTED: str
    :cvar REVIEWED: "REVIEWED"
    :vartype REVIEWED: str
    :cvar CONFLICTRESOLVED: "CONFLICT_RESOLVED"
    :vartype CONFLICTRESOLVED: str
    :cvar MERGED: "MERGED"
    :vartype MERGED: str
    :cvar REVERTED: "REVERTED"
    :vartype REVERTED: str
    """

    DRAFTED = "DRAFTED"
    REVIEWED = "REVIEWED"
    CONFLICTRESOLVED = "CONFLICT_RESOLVED"
    MERGED = "MERGED"
    REVERTED = "REVERTED"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, MergeRequestDetailStage._member_map_.values())
        )


@JsonMap(
    {
        "change_type": "changeType",
        "component_guid": "componentGuid",
        "created_by": "createdBy",
        "created_date": "createdDate",
        "destination_revision": "destinationRevision",
        "locked_on_destination_branch": "lockedOnDestinationBranch",
        "merge_revision": "mergeRevision",
        "modified_by": "modifiedBy",
        "modified_date": "modifiedDate",
        "source_revision": "sourceRevision",
    }
)
class MergeRequestDetail(BaseModel):
    """MergeRequestDetail

    :param change_type: change_type, defaults to None
    :type change_type: ChangeType, optional
    :param component_guid: component_guid, defaults to None
    :type component_guid: str, optional
    :param conflict: conflict, defaults to None
    :type conflict: bool, optional
    :param created_by: created_by, defaults to None
    :type created_by: str, optional
    :param created_date: created_date, defaults to None
    :type created_date: str, optional
    :param destination_revision: destination_revision, defaults to None
    :type destination_revision: int, optional
    :param excluded: When true, signifies that a component will not be included in the Merge., defaults to None
    :type excluded: bool, optional
    :param locked_on_destination_branch: locked_on_destination_branch, defaults to None
    :type locked_on_destination_branch: bool, optional
    :param merge_revision: merge_revision, defaults to None
    :type merge_revision: int, optional
    :param modified_by: modified_by, defaults to None
    :type modified_by: str, optional
    :param modified_date: modified_date, defaults to None
    :type modified_date: str, optional
    :param resolution: resolution, defaults to None
    :type resolution: Resolution, optional
    :param source_revision: source_revision, defaults to None
    :type source_revision: int, optional
    :param stage: stage, defaults to None
    :type stage: MergeRequestDetailStage, optional
    """

    def __init__(
        self,
        change_type: ChangeType = SENTINEL,
        component_guid: str = SENTINEL,
        conflict: bool = SENTINEL,
        created_by: str = SENTINEL,
        created_date: str = SENTINEL,
        destination_revision: int = SENTINEL,
        excluded: bool = SENTINEL,
        locked_on_destination_branch: bool = SENTINEL,
        merge_revision: int = SENTINEL,
        modified_by: str = SENTINEL,
        modified_date: str = SENTINEL,
        resolution: Resolution = SENTINEL,
        source_revision: int = SENTINEL,
        stage: MergeRequestDetailStage = SENTINEL,
        **kwargs
    ):
        """MergeRequestDetail

        :param change_type: change_type, defaults to None
        :type change_type: ChangeType, optional
        :param component_guid: component_guid, defaults to None
        :type component_guid: str, optional
        :param conflict: conflict, defaults to None
        :type conflict: bool, optional
        :param created_by: created_by, defaults to None
        :type created_by: str, optional
        :param created_date: created_date, defaults to None
        :type created_date: str, optional
        :param destination_revision: destination_revision, defaults to None
        :type destination_revision: int, optional
        :param excluded: When true, signifies that a component will not be included in the Merge., defaults to None
        :type excluded: bool, optional
        :param locked_on_destination_branch: locked_on_destination_branch, defaults to None
        :type locked_on_destination_branch: bool, optional
        :param merge_revision: merge_revision, defaults to None
        :type merge_revision: int, optional
        :param modified_by: modified_by, defaults to None
        :type modified_by: str, optional
        :param modified_date: modified_date, defaults to None
        :type modified_date: str, optional
        :param resolution: resolution, defaults to None
        :type resolution: Resolution, optional
        :param source_revision: source_revision, defaults to None
        :type source_revision: int, optional
        :param stage: stage, defaults to None
        :type stage: MergeRequestDetailStage, optional
        """
        if change_type is not SENTINEL:
            self.change_type = self._enum_matching(
                change_type, ChangeType.list(), "change_type"
            )
        if component_guid is not SENTINEL:
            self.component_guid = component_guid
        if conflict is not SENTINEL:
            self.conflict = conflict
        if created_by is not SENTINEL:
            self.created_by = created_by
        if created_date is not SENTINEL:
            self.created_date = created_date
        if destination_revision is not SENTINEL:
            self.destination_revision = destination_revision
        if excluded is not SENTINEL:
            self.excluded = excluded
        if locked_on_destination_branch is not SENTINEL:
            self.locked_on_destination_branch = locked_on_destination_branch
        if merge_revision is not SENTINEL:
            self.merge_revision = merge_revision
        if modified_by is not SENTINEL:
            self.modified_by = modified_by
        if modified_date is not SENTINEL:
            self.modified_date = modified_date
        if resolution is not SENTINEL:
            self.resolution = self._enum_matching(
                resolution, Resolution.list(), "resolution"
            )
        if source_revision is not SENTINEL:
            self.source_revision = source_revision
        if stage is not SENTINEL:
            self.stage = self._enum_matching(
                stage, MergeRequestDetailStage.list(), "stage"
            )
        self._kwargs = kwargs




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..account_user_role import AccountUserRoleService
from ...models import (
    AccountUserRole,
    AccountUserRoleQueryResponse,
    AccountUserRoleQueryConfig,
)


class AccountUserRoleServiceAsync(AccountUserRoleService):
    """
    Async Wrapper for AccountUserRoleServiceAsync
    """

    def create_account_user_role(
        self, request_body: AccountUserRole = None
    ) -> Awaitable[Union[AccountUserRole, str]]:
        return to_async(super().create_account_user_role)(request_body)

    def query_account_user_role(
        self, request_body: AccountUserRoleQueryConfig = None
    ) -> Awaitable[Union[AccountUserRoleQueryResponse, str]]:
        return to_async(super().query_account_user_role)(request_body)

    def query_more_account_user_role(
        self, request_body: str
    ) -> Awaitable[Union[AccountUserRoleQueryResponse, str]]:
        return to_async(super().query_more_account_user_role)(request_body)

    def delete_account_user_role(self, id_: str) -> Awaitable[None]:
        return to_async(super().delete_account_user_role)(id_)




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .integration_pack_environment_attachment_expression import (
    IntegrationPackEnvironmentAttachmentExpression,
    IntegrationPackEnvironmentAttachmentExpressionGuard,
)
from .integration_pack_environment_attachment_simple_expression import (
    IntegrationPackEnvironmentAttachmentSimpleExpression,
)
from .integration_pack_environment_attachment_grouping_expression import (
    IntegrationPackEnvironmentAttachmentGroupingExpression,
)


@JsonMap({})
class IntegrationPackEnvironmentAttachmentQueryConfigQueryFilter(BaseModel):
    """IntegrationPackEnvironmentAttachmentQueryConfigQueryFilter

    :param expression: expression
    :type expression: IntegrationPackEnvironmentAttachmentExpression
    """

    def __init__(
        self, expression: IntegrationPackEnvironmentAttachmentExpression, **kwargs
    ):
        """IntegrationPackEnvironmentAttachmentQueryConfigQueryFilter

        :param expression: expression
        :type expression: IntegrationPackEnvironmentAttachmentExpression
        """
        self.expression = (
            IntegrationPackEnvironmentAttachmentExpressionGuard.return_one_of(
                expression
            )
        )
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class IntegrationPackEnvironmentAttachmentQueryConfig(BaseModel):
    """IntegrationPackEnvironmentAttachmentQueryConfig

    :param query_filter: query_filter
    :type query_filter: IntegrationPackEnvironmentAttachmentQueryConfigQueryFilter
    """

    def __init__(
        self,
        query_filter: IntegrationPackEnvironmentAttachmentQueryConfigQueryFilter,
        **kwargs,
    ):
        """IntegrationPackEnvironmentAttachmentQueryConfig

        :param query_filter: query_filter
        :type query_filter: IntegrationPackEnvironmentAttachmentQueryConfigQueryFilter
        """
        self.query_filter = self._define_object(
            query_filter, IntegrationPackEnvironmentAttachmentQueryConfigQueryFilter
        )
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .shared_communication_channel_component_expression import (
    SharedCommunicationChannelComponentExpression,
    SharedCommunicationChannelComponentExpressionGuard,
)
from .shared_communication_channel_component_simple_expression import (
    SharedCommunicationChannelComponentSimpleExpression,
)
from .shared_communication_channel_component_grouping_expression import (
    SharedCommunicationChannelComponentGroupingExpression,
)


@JsonMap({})
class SharedCommunicationChannelComponentQueryConfigQueryFilter(BaseModel):
    """SharedCommunicationChannelComponentQueryConfigQueryFilter

    :param expression: expression
    :type expression: SharedCommunicationChannelComponentExpression
    """

    def __init__(
        self, expression: SharedCommunicationChannelComponentExpression, **kwargs
    ):
        """SharedCommunicationChannelComponentQueryConfigQueryFilter

        :param expression: expression
        :type expression: SharedCommunicationChannelComponentExpression
        """
        self.expression = (
            SharedCommunicationChannelComponentExpressionGuard.return_one_of(expression)
        )
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class SharedCommunicationChannelComponentQueryConfig(BaseModel):
    """SharedCommunicationChannelComponentQueryConfig

    :param query_filter: query_filter
    :type query_filter: SharedCommunicationChannelComponentQueryConfigQueryFilter
    """

    def __init__(
        self,
        query_filter: SharedCommunicationChannelComponentQueryConfigQueryFilter,
        **kwargs,
    ):
        """SharedCommunicationChannelComponentQueryConfig

        :param query_filter: query_filter
        :type query_filter: SharedCommunicationChannelComponentQueryConfigQueryFilter
        """
        self.query_filter = self._define_object(
            query_filter, SharedCommunicationChannelComponentQueryConfigQueryFilter
        )
        self._kwargs = kwargs




from enum import Enum
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


class DeployedExpiredCertificateSimpleExpressionOperator(Enum):
    """An enumeration representing different categories.

    :cvar LESSTHANOREQUAL: "LESS_THAN_OR_EQUAL"
    :vartype LESSTHANOREQUAL: str
    """

    LESSTHANOREQUAL = "LESS_THAN_OR_EQUAL"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                DeployedExpiredCertificateSimpleExpressionOperator._member_map_.values(),
            )
        )


class DeployedExpiredCertificateSimpleExpressionProperty(Enum):
    """An enumeration representing different categories.

    :cvar CONTAINERID: "containerId"
    :vartype CONTAINERID: str
    :cvar CONTAINERNAME: "containerName"
    :vartype CONTAINERNAME: str
    :cvar ENVIRONMENTID: "environmentId"
    :vartype ENVIRONMENTID: str
    :cvar ENVIRONMENTNAME: "environmentName"
    :vartype ENVIRONMENTNAME: str
    :cvar EXPIRATIONBOUNDARY: "expirationBoundary"
    :vartype EXPIRATIONBOUNDARY: str
    """

    CONTAINERID = "containerId"
    CONTAINERNAME = "containerName"
    ENVIRONMENTID = "environmentId"
    ENVIRONMENTNAME = "environmentName"
    EXPIRATIONBOUNDARY = "expirationBoundary"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                DeployedExpiredCertificateSimpleExpressionProperty._member_map_.values(),
            )
        )


@JsonMap({})
class DeployedExpiredCertificateSimpleExpression(BaseModel):
    """DeployedExpiredCertificateSimpleExpression

    :param argument: argument, defaults to None
    :type argument: List[str], optional
    :param operator: operator
    :type operator: DeployedExpiredCertificateSimpleExpressionOperator
    :param property: property
    :type property: DeployedExpiredCertificateSimpleExpressionProperty
    """

    def __init__(
        self,
        operator: DeployedExpiredCertificateSimpleExpressionOperator,
        property: DeployedExpiredCertificateSimpleExpressionProperty,
        argument: List[str] = SENTINEL,
        **kwargs
    ):
        """DeployedExpiredCertificateSimpleExpression

        :param argument: argument, defaults to None
        :type argument: List[str], optional
        :param operator: operator
        :type operator: DeployedExpiredCertificateSimpleExpressionOperator
        :param property: property
        :type property: DeployedExpiredCertificateSimpleExpressionProperty
        """
        if argument is not SENTINEL:
            self.argument = argument
        self.operator = self._enum_matching(
            operator,
            DeployedExpiredCertificateSimpleExpressionOperator.list(),
            "operator",
        )
        self.property = self._enum_matching(
            property,
            DeployedExpiredCertificateSimpleExpressionProperty.list(),
            "property",
        )
        self._kwargs = kwargs




from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..net.transport.utils import parse_xml_to_dict
from ..models import (
    DeployedExpiredCertificateQueryConfig,
    DeployedExpiredCertificateQueryResponse,
)


class DeployedExpiredCertificateService(BaseService):

    @cast_models
    def query_deployed_expired_certificate(
        self, request_body: DeployedExpiredCertificateQueryConfig = None
    ) -> Union[DeployedExpiredCertificateQueryResponse, str]:
        """If a QUERY omits an explicit timespan filter — that is, the query does not use `expirationBoundary` in an expression — it applies a default timespan filter using the value of 30 and the LESS_THAN operator. This filter specifies expired certificates and certificates expiring within 30 days.

         For general information about the structure of QUERY filters, their sample payloads, and how to handle the paged results, refer to [Query filters](#section/Introduction/Query-filters) and [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body., defaults to None
        :type request_body: DeployedExpiredCertificateQueryConfig, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[DeployedExpiredCertificateQueryResponse, str]
        """

        Validator(DeployedExpiredCertificateQueryConfig).is_optional().validate(
            request_body
        )

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/DeployedExpiredCertificate/query",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return DeployedExpiredCertificateQueryResponse._unmap(response)
        if content == "application/xml":
            return DeployedExpiredCertificateQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_more_deployed_expired_certificate(
        self, request_body: str
    ) -> Union[DeployedExpiredCertificateQueryResponse, str]:
        """To learn about using `queryMore`, refer to [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body.
        :type request_body: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[DeployedExpiredCertificateQueryResponse, str]
        """

        Validator(str).validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/DeployedExpiredCertificate/queryMore",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body, "text/plain")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return DeployedExpiredCertificateQueryResponse._unmap(response)
        if content == "application/xml":
            return DeployedExpiredCertificateQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .deployed_package_simple_expression import DeployedPackageSimpleExpression
from .deployed_package_grouping_expression import DeployedPackageGroupingExpression


class DeployedPackageExpressionGuard(OneOfBaseModel):
    class_list = {
        "DeployedPackageSimpleExpression": DeployedPackageSimpleExpression,
        "DeployedPackageGroupingExpression": DeployedPackageGroupingExpression,
    }


DeployedPackageExpression = Union[
    DeployedPackageSimpleExpression, DeployedPackageGroupingExpression
]




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .privilege import Privilege


@JsonMap({"privilege": "Privilege"})
class Privileges(BaseModel):
    """One or more privileges assigned to the role.

    :param privilege: privilege, defaults to None
    :type privilege: List[Privilege], optional
    """

    def __init__(self, privilege: List[Privilege] = SENTINEL, **kwargs):
        """One or more privileges assigned to the role.

        :param privilege: privilege, defaults to None
        :type privilege: List[Privilege], optional
        """
        if privilege is not SENTINEL:
            self.privilege = self._define_list(privilege, Privilege)
        self._kwargs = kwargs




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap(
    {
        "gcp_client_email": "gcpClientEmail",
        "gcp_client_id": "gcpClientId",
        "gcp_private_key": "gcpPrivateKey",
        "gcp_private_key_id": "gcpPrivateKeyId",
        "gcp_project_id": "gcpProjectId",
    }
)
class CloudAttachmentGcpServiceAccountConfig(BaseModel):
    """CloudAttachmentGcpServiceAccountConfig

    :param gcp_client_email: gcp_client_email
    :type gcp_client_email: str
    :param gcp_client_id: gcp_client_id
    :type gcp_client_id: str
    :param gcp_private_key: gcp_private_key
    :type gcp_private_key: str
    :param gcp_private_key_id: gcp_private_key_id
    :type gcp_private_key_id: str
    :param gcp_project_id: gcp_project_id
    :type gcp_project_id: str
    """

    def __init__(
        self,
        gcp_client_email: str,
        gcp_client_id: str,
        gcp_private_key: str,
        gcp_private_key_id: str,
        gcp_project_id: str,
        **kwargs,
    ):
        """CloudAttachmentGcpServiceAccountConfig

        :param gcp_client_email: gcp_client_email
        :type gcp_client_email: str
        :param gcp_client_id: gcp_client_id
        :type gcp_client_id: str
        :param gcp_private_key: gcp_private_key
        :type gcp_private_key: str
        :param gcp_private_key_id: gcp_private_key_id
        :type gcp_private_key_id: str
        :param gcp_project_id: gcp_project_id
        :type gcp_project_id: str
        """
        self.gcp_client_email = gcp_client_email
        self.gcp_client_id = gcp_client_id
        self.gcp_private_key = gcp_private_key
        self.gcp_private_key_id = gcp_private_key_id
        self.gcp_project_id = gcp_project_id
        self._kwargs = kwargs




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .cross_reference_row import CrossReferenceRow


@JsonMap({})
class CrossReferenceRows(BaseModel):
    """CrossReferenceRows

    :param row: row, defaults to None
    :type row: List[CrossReferenceRow], optional
    """

    def __init__(self, row: List[CrossReferenceRow] = SENTINEL, **kwargs):
        """CrossReferenceRows

        :param row: row, defaults to None
        :type row: List[CrossReferenceRow], optional
        """
        if row is not SENTINEL:
            self.row = self._define_list(row, CrossReferenceRow)
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .mllp_send_settings import MllpSendSettings


@JsonMap({"mllp_send_settings": "MLLPSendSettings"})
class MllpCommunicationOptions(BaseModel):
    """MllpCommunicationOptions

    :param mllp_send_settings: mllp_send_settings, defaults to None
    :type mllp_send_settings: MllpSendSettings, optional
    """

    def __init__(self, mllp_send_settings: MllpSendSettings = SENTINEL, **kwargs):
        """MllpCommunicationOptions

        :param mllp_send_settings: mllp_send_settings, defaults to None
        :type mllp_send_settings: MllpSendSettings, optional
        """
        if mllp_send_settings is not SENTINEL:
            self.mllp_send_settings = self._define_object(
                mllp_send_settings, MllpSendSettings
            )
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .execution_count_account_expression import (
    ExecutionCountAccountExpression,
    ExecutionCountAccountExpressionGuard,
)
from .execution_count_account_simple_expression import (
    ExecutionCountAccountSimpleExpression,
)
from .execution_count_account_grouping_expression import (
    ExecutionCountAccountGroupingExpression,
)


@JsonMap({})
class ExecutionCountAccountQueryConfigQueryFilter(BaseModel):
    """ExecutionCountAccountQueryConfigQueryFilter

    :param expression: expression
    :type expression: ExecutionCountAccountExpression
    """

    def __init__(self, expression: ExecutionCountAccountExpression, **kwargs):
        """ExecutionCountAccountQueryConfigQueryFilter

        :param expression: expression
        :type expression: ExecutionCountAccountExpression
        """
        self.expression = ExecutionCountAccountExpressionGuard.return_one_of(expression)
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class ExecutionCountAccountQueryConfig(BaseModel):
    """ExecutionCountAccountQueryConfig

    :param query_filter: query_filter
    :type query_filter: ExecutionCountAccountQueryConfigQueryFilter
    """

    def __init__(
        self, query_filter: ExecutionCountAccountQueryConfigQueryFilter, **kwargs
    ):
        """ExecutionCountAccountQueryConfig

        :param query_filter: query_filter
        :type query_filter: ExecutionCountAccountQueryConfigQueryFilter
        """
        self.query_filter = self._define_object(
            query_filter, ExecutionCountAccountQueryConfigQueryFilter
        )
        self._kwargs = kwargs




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .shared_web_server_login_module_option import SharedWebServerLoginModuleOption


@JsonMap({"login_module": "loginModule"})
class SharedWebServerLoginModuleConfiguration(BaseModel):
    """SharedWebServerLoginModuleConfiguration

    :param login_module: login_module, defaults to None
    :type login_module: List[SharedWebServerLoginModuleOption], optional
    """

    def __init__(
        self, login_module: List[SharedWebServerLoginModuleOption] = SENTINEL, **kwargs
    ):
        """SharedWebServerLoginModuleConfiguration

        :param login_module: login_module, defaults to None
        :type login_module: List[SharedWebServerLoginModuleOption], optional
        """
        if login_module is not SENTINEL:
            self.login_module = self._define_list(
                login_module, SharedWebServerLoginModuleOption
            )
        self._kwargs = kwargs




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..cloud_attachment_properties import (
    CloudAttachmentPropertiesService,
)
from ...models import (
    CloudAttachmentProperties,
    AsyncOperationTokenResult,
    CloudAttachmentPropertiesAsyncResponse,
)


class CloudAttachmentPropertiesServiceAsync(
    CloudAttachmentPropertiesService
):
    """
    Async Wrapper for CloudAttachmentPropertiesServiceAsync
    """

    def update_cloud_attachment_properties(
        self, id_: str, request_body: CloudAttachmentProperties
    ) -> Awaitable[Union[CloudAttachmentProperties, str]]:
        return to_async(super().update_cloud_attachment_properties)(
            id_, request_body
        )

    def async_get_cloud_attachment_properties(
        self, id_: str
    ) -> Awaitable[Union[AsyncOperationTokenResult, str]]:
        return to_async(super().async_get_cloud_attachment_properties)(id_)

    def async_token_cloud_attachment_properties(
        self, token: str
    ) -> Awaitable[Union[CloudAttachmentPropertiesAsyncResponse, str]]:
        return to_async(super().async_token_cloud_attachment_properties)(token)




from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..net.transport.utils import parse_xml_to_dict
from ..models import (
    TradingPartnerProcessingGroup,
    TradingPartnerProcessingGroupBulkRequest,
    TradingPartnerProcessingGroupBulkResponse,
    TradingPartnerProcessingGroupQueryConfig,
    TradingPartnerProcessingGroupQueryResponse,
)


class TradingPartnerProcessingGroupService(BaseService):

    @cast_models
    def create_trading_partner_processing_group(
        self, request_body: TradingPartnerProcessingGroup = None
    ) -> Union[TradingPartnerProcessingGroup, str]:
        """Send an HTTP POST request where `accountId` is the ID of the authenticating account for the request.
         If you omit the folderName field, you must include the folderId field — and vice versa.

        :param request_body: The request body., defaults to None
        :type request_body: TradingPartnerProcessingGroup, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[TradingPartnerProcessingGroup, str]
        """

        Validator(TradingPartnerProcessingGroup).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/TradingPartnerProcessingGroup",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return TradingPartnerProcessingGroup._unmap(response)
        if content == "application/xml":
            return TradingPartnerProcessingGroup._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def get_trading_partner_processing_group(
        self, id_: str
    ) -> Union[TradingPartnerProcessingGroup, str]:
        """The ordinary GET operation returns a single Trading Partner Processing Group object based on the supplied ID. The bulk GET operation returns multiple Trading Partner Processing Group objects based on the supplied IDs, to a maximum of 100.
         A GET operation specifying the ID of a deleted processing group component retrieves the component. In the component, the deleted field’s value is true.

        :param id_: id_
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[TradingPartnerProcessingGroup, str]
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/TradingPartnerProcessingGroup/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return TradingPartnerProcessingGroup._unmap(response)
        if content == "application/xml":
            return TradingPartnerProcessingGroup._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def update_trading_partner_processing_group(
        self, id_: str, request_body: TradingPartnerProcessingGroup = None
    ) -> Union[TradingPartnerProcessingGroup, str]:
        """An UPDATE operation specifying the ID of a deleted processing group component restores the component to a non-deleted state, assuming the request is otherwise valid. It also overwrites the entire processing group component.

        :param request_body: The request body., defaults to None
        :type request_body: TradingPartnerProcessingGroup, optional
        :param id_: id_
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[TradingPartnerProcessingGroup, str]
        """

        Validator(TradingPartnerProcessingGroup).is_optional().validate(request_body)
        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/TradingPartnerProcessingGroup/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return TradingPartnerProcessingGroup._unmap(response)
        if content == "application/xml":
            return TradingPartnerProcessingGroup._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def delete_trading_partner_processing_group(self, id_: str) -> None:
        """A DELETE operation specifying the ID of a deleted processing group component returns a false response. If you deleted the component successfully, the response is "true".

        :param id_: id_
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/TradingPartnerProcessingGroup/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("DELETE")
        )

        response, status, _ = self.send_request(serialized_request)

    @cast_models
    def bulk_trading_partner_processing_group(
        self, request_body: TradingPartnerProcessingGroupBulkRequest = None
    ) -> Union[TradingPartnerProcessingGroupBulkResponse, str]:
        """To learn more about `bulk`, refer to [Bulk GET operations](#section/Introduction/Bulk-GET-operations).

        :param request_body: The request body., defaults to None
        :type request_body: TradingPartnerProcessingGroupBulkRequest, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[TradingPartnerProcessingGroupBulkResponse, str]
        """

        Validator(TradingPartnerProcessingGroupBulkRequest).is_optional().validate(
            request_body
        )

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/TradingPartnerProcessingGroup/bulk",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return TradingPartnerProcessingGroupBulkResponse._unmap(response)
        if content == "application/xml":
            return TradingPartnerProcessingGroupBulkResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_trading_partner_processing_group(
        self, request_body: TradingPartnerProcessingGroupQueryConfig = None
    ) -> Union[TradingPartnerProcessingGroupQueryResponse, str]:
        """The QUERY operation returns all of the authenticating account’s processing group components. The operation does not return full component representations; it returns, for each result, the component’s name, ID, and folder ID.

         For general information about the structure of QUERY filters, their sample payloads, and how to handle the paged results, refer to [Query filters](#section/Introduction/Query-filters) and [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body., defaults to None
        :type request_body: TradingPartnerProcessingGroupQueryConfig, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[TradingPartnerProcessingGroupQueryResponse, str]
        """

        Validator(TradingPartnerProcessingGroupQueryConfig).is_optional().validate(
            request_body
        )

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/TradingPartnerProcessingGroup/query",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return TradingPartnerProcessingGroupQueryResponse._unmap(response)
        if content == "application/xml":
            return TradingPartnerProcessingGroupQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_more_trading_partner_processing_group(
        self, request_body: str
    ) -> Union[TradingPartnerProcessingGroupQueryResponse, str]:
        """To learn about using `queryMore`, refer to [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body.
        :type request_body: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[TradingPartnerProcessingGroupQueryResponse, str]
        """

        Validator(str).validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/TradingPartnerProcessingGroup/queryMore",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body, "text/plain")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return TradingPartnerProcessingGroupQueryResponse._unmap(response)
        if content == "application/xml":
            return TradingPartnerProcessingGroupQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)




from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


class ApiType(Enum):
    """An enumeration representing different categories.

    :cvar BASIC: "basic"
    :vartype BASIC: str
    :cvar INTERMEDIATE: "intermediate"
    :vartype INTERMEDIATE: str
    :cvar ADVANCED: "advanced"
    :vartype ADVANCED: str
    """

    BASIC = "basic"
    INTERMEDIATE = "intermediate"
    ADVANCED = "advanced"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, ApiType._member_map_.values()))


class Auth(Enum):
    """An enumeration representing different categories.

    :cvar NONE: "none"
    :vartype NONE: str
    :cvar BASIC: "basic"
    :vartype BASIC: str
    """

    NONE = "none"
    BASIC = "basic"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, Auth._member_map_.values()))


class MinAuth(Enum):
    """An enumeration representing different categories.

    :cvar NONE: "none"
    :vartype NONE: str
    :cvar BASIC: "basic"
    :vartype BASIC: str
    """

    NONE = "none"
    BASIC = "basic"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, MinAuth._member_map_.values()))


@JsonMap(
    {
        "api_type": "apiType",
        "atom_id": "atomId",
        "auth_token": "authToken",
        "check_forwarded_headers": "checkForwardedHeaders",
        "external_host": "externalHost",
        "external_http_port": "externalHttpPort",
        "external_https_port": "externalHttpsPort",
        "http_port": "httpPort",
        "https_port": "httpsPort",
        "internal_host": "internalHost",
        "max_threads": "maxThreads",
        "min_auth": "minAuth",
        "override_url": "overrideUrl",
        "ssl_certificate_id": "sslCertificateId",
    }
)
class SharedServerInformation(BaseModel):
    """SharedServerInformation

    :param api_type: The level of user management and API management functionality applicable to the shared web server.Options are basic, intermediate, and advanced. The default is intermediate., defaults to None
    :type api_type: ApiType, optional
    :param atom_id: The ID of the Runtime that is hosting the shared web server., defaults to None
    :type atom_id: str, optional
    :param auth: The authentication required by the web server. Options are none and basic. If minAuth is set to basic, you must also set auth to basic., defaults to None
    :type auth: Auth, optional
    :param auth_token: If you configure BASIC authentication, this is an authentication token for connecting to the shared web server. You cannot update this with the UPDATE operation., defaults to None
    :type auth_token: str, optional
    :param check_forwarded_headers: Information regarding the external host, might be forwarded in headers. The embedded Java technology is capable of examining these headers and extracting external host information for response and callback purposes. Set this to true to enable the server to check for this information. The default is false., defaults to None
    :type check_forwarded_headers: bool, optional
    :param external_host: The external host name or IP for the listener., defaults to None
    :type external_host: str, optional
    :param external_http_port: The external HTTP port routes to the shared web server listener., defaults to None
    :type external_http_port: int, optional
    :param external_https_port: The external HTTPS port routes to the shared web server listener., defaults to None
    :type external_https_port: int, optional
    :param http_port: The HTTP port on which the web server listens. The default port is 9090., defaults to None
    :type http_port: int, optional
    :param https_port: The SSL \(HTTPS\) port on which the web server listens, if applicable. The default port is 9093., defaults to None
    :type https_port: int, optional
    :param internal_host: For multi-homed boxes, the IP address you want to use for binding to a specific interface., defaults to None
    :type internal_host: str, optional
    :param max_threads: The maximum number of handler threads that the listen process spawn. It queues other requests., defaults to None
    :type max_threads: int, optional
    :param min_auth: The minimum authentication required by the web server. Options are none and basic. The are multi-tenant, so the default is set to basic. The default for local Runtimes and Runtime clusters is none., defaults to None
    :type min_auth: MinAuth, optional
    :param override_url: Allows manual overriding of the exposed URL used to connect to the shared web server. This value is for informational purposes for any tenant. By default, this is false, meaning the URL is constructed based on the host name or external host name and port or SSL port settings. Set this to true to specify a custom URL attribute value., defaults to None
    :type override_url: bool, optional
    :param ssl_certificate_id: The component ID for the SSL certificate used by the server, if applicable., defaults to None
    :type ssl_certificate_id: str, optional
    :param url: The URL for connecting to the shared web server., defaults to None
    :type url: str, optional
    """

    def __init__(
        self,
        api_type: ApiType = SENTINEL,
        atom_id: str = SENTINEL,
        auth: Auth = SENTINEL,
        auth_token: str = SENTINEL,
        check_forwarded_headers: bool = SENTINEL,
        external_host: str = SENTINEL,
        external_http_port: int = SENTINEL,
        external_https_port: int = SENTINEL,
        http_port: int = SENTINEL,
        https_port: int = SENTINEL,
        internal_host: str = SENTINEL,
        max_threads: int = SENTINEL,
        min_auth: MinAuth = SENTINEL,
        override_url: bool = SENTINEL,
        ssl_certificate_id: str = SENTINEL,
        url: str = SENTINEL,
        **kwargs
    ):
        """SharedServerInformation

        :param api_type: The level of user management and API management functionality applicable to the shared web server.Options are basic, intermediate, and advanced. The default is intermediate., defaults to None
        :type api_type: ApiType, optional
        :param atom_id: The ID of the Runtime that is hosting the shared web server., defaults to None
        :type atom_id: str, optional
        :param auth: The authentication required by the web server. Options are none and basic. If minAuth is set to basic, you must also set auth to basic., defaults to None
        :type auth: Auth, optional
        :param auth_token: If you configure BASIC authentication, this is an authentication token for connecting to the shared web server. You cannot update this with the UPDATE operation., defaults to None
        :type auth_token: str, optional
        :param check_forwarded_headers: Information regarding the external host, might be forwarded in headers. The embedded Java technology is capable of examining these headers and extracting external host information for response and callback purposes. Set this to true to enable the server to check for this information. The default is false., defaults to None
        :type check_forwarded_headers: bool, optional
        :param external_host: The external host name or IP for the listener., defaults to None
        :type external_host: str, optional
        :param external_http_port: The external HTTP port routes to the shared web server listener., defaults to None
        :type external_http_port: int, optional
        :param external_https_port: The external HTTPS port routes to the shared web server listener., defaults to None
        :type external_https_port: int, optional
        :param http_port: The HTTP port on which the web server listens. The default port is 9090., defaults to None
        :type http_port: int, optional
        :param https_port: The SSL \(HTTPS\) port on which the web server listens, if applicable. The default port is 9093., defaults to None
        :type https_port: int, optional
        :param internal_host: For multi-homed boxes, the IP address you want to use for binding to a specific interface., defaults to None
        :type internal_host: str, optional
        :param max_threads: The maximum number of handler threads that the listen process spawn. It queues other requests., defaults to None
        :type max_threads: int, optional
        :param min_auth: The minimum authentication required by the web server. Options are none and basic. The are multi-tenant, so the default is set to basic. The default for local Runtimes and Runtime clusters is none., defaults to None
        :type min_auth: MinAuth, optional
        :param override_url: Allows manual overriding of the exposed URL used to connect to the shared web server. This value is for informational purposes for any tenant. By default, this is false, meaning the URL is constructed based on the host name or external host name and port or SSL port settings. Set this to true to specify a custom URL attribute value., defaults to None
        :type override_url: bool, optional
        :param ssl_certificate_id: The component ID for the SSL certificate used by the server, if applicable., defaults to None
        :type ssl_certificate_id: str, optional
        :param url: The URL for connecting to the shared web server., defaults to None
        :type url: str, optional
        """
        if api_type is not SENTINEL:
            self.api_type = self._enum_matching(api_type, ApiType.list(), "api_type")
        if atom_id is not SENTINEL:
            self.atom_id = atom_id
        if auth is not SENTINEL:
            self.auth = self._enum_matching(auth, Auth.list(), "auth")
        if auth_token is not SENTINEL:
            self.auth_token = auth_token
        if check_forwarded_headers is not SENTINEL:
            self.check_forwarded_headers = check_forwarded_headers
        if external_host is not SENTINEL:
            self.external_host = external_host
        if external_http_port is not SENTINEL:
            self.external_http_port = external_http_port
        if external_https_port is not SENTINEL:
            self.external_https_port = external_https_port
        if http_port is not SENTINEL:
            self.http_port = http_port
        if https_port is not SENTINEL:
            self.https_port = https_port
        if internal_host is not SENTINEL:
            self.internal_host = internal_host
        if max_threads is not SENTINEL:
            self.max_threads = max_threads
        if min_auth is not SENTINEL:
            self.min_auth = self._enum_matching(min_auth, MinAuth.list(), "min_auth")
        if override_url is not SENTINEL:
            self.override_url = override_url
        if ssl_certificate_id is not SENTINEL:
            self.ssl_certificate_id = ssl_certificate_id
        if url is not SENTINEL:
            self.url = url
        self._kwargs = kwargs




from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


class CustomTrackedFieldType(Enum):
    """An enumeration representing different categories.

    :cvar CHARACTER: "character"
    :vartype CHARACTER: str
    :cvar DATETIME: "datetime"
    :vartype DATETIME: str
    :cvar NUMBER: "number"
    :vartype NUMBER: str
    """

    CHARACTER = "character"
    DATETIME = "datetime"
    NUMBER = "number"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, CustomTrackedFieldType._member_map_.values())
        )


@JsonMap({"type_": "type"})
class CustomTrackedField(BaseModel):
    """CustomTrackedField

    :param label: The display name of the custom tracked field., defaults to None
    :type label: str, optional
    :param position: The display position of the custom tracked field., defaults to None
    :type position: int, optional
    :param type_: The type of custom tracked field. Allowed values include character, datetime, and number., defaults to None
    :type type_: CustomTrackedFieldType, optional
    """

    def __init__(
        self,
        label: str = SENTINEL,
        position: int = SENTINEL,
        type_: CustomTrackedFieldType = SENTINEL,
        **kwargs
    ):
        """CustomTrackedField

        :param label: The display name of the custom tracked field., defaults to None
        :type label: str, optional
        :param position: The display position of the custom tracked field., defaults to None
        :type position: int, optional
        :param type_: The type of custom tracked field. Allowed values include character, datetime, and number., defaults to None
        :type type_: CustomTrackedFieldType, optional
        """
        if label is not SENTINEL:
            self.label = label
        if position is not SENTINEL:
            self.position = position
        if type_ is not SENTINEL:
            self.type_ = self._enum_matching(
                type_, CustomTrackedFieldType.list(), "type_"
            )
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .field_summary import FieldSummary


@JsonMap(
    {
        "atom_id": "atomId",
        "connection_id": "connectionId",
        "extension_group_id": "extensionGroupId",
        "id_": "id",
        "process_id": "processId",
    }
)
class AtomConnectionFieldExtensionSummary(BaseModel):
    """AtomConnectionFieldExtensionSummary

    :param atom_id: The ID of the Runtime., defaults to None
    :type atom_id: str, optional
    :param connection_id: The ID of the connection., defaults to None
    :type connection_id: str, optional
    :param extension_group_id: If the process is in a multi-install integration pack, this is the ID of the multi-install integration pack, which is the same as the ID of the process., defaults to None
    :type extension_group_id: str, optional
    :param field: field
    :type field: FieldSummary
    :param id_: The ID of the object. This is a conceptual ID synthesized from the IDs of the\<br /\> -   process\<br /\>-   connection\<br /\> -   multi-install integration pack \(extensionGroupId\), if applicable\<br /\>-   Atom, defaults to None
    :type id_: str, optional
    :param process_id: The ID of the process., defaults to None
    :type process_id: str, optional
    """

    def __init__(
        self,
        field: FieldSummary,
        atom_id: str = SENTINEL,
        connection_id: str = SENTINEL,
        extension_group_id: str = SENTINEL,
        id_: str = SENTINEL,
        process_id: str = SENTINEL,
        **kwargs,
    ):
        """AtomConnectionFieldExtensionSummary

        :param atom_id: The ID of the Runtime., defaults to None
        :type atom_id: str, optional
        :param connection_id: The ID of the connection., defaults to None
        :type connection_id: str, optional
        :param extension_group_id: If the process is in a multi-install integration pack, this is the ID of the multi-install integration pack, which is the same as the ID of the process., defaults to None
        :type extension_group_id: str, optional
        :param field: field
        :type field: FieldSummary
        :param id_: The ID of the object. This is a conceptual ID synthesized from the IDs of the\<br /\> -   process\<br /\>-   connection\<br /\> -   multi-install integration pack \(extensionGroupId\), if applicable\<br /\>-   Atom, defaults to None
        :type id_: str, optional
        :param process_id: The ID of the process., defaults to None
        :type process_id: str, optional
        """
        if atom_id is not SENTINEL:
            self.atom_id = atom_id
        if connection_id is not SENTINEL:
            self.connection_id = connection_id
        if extension_group_id is not SENTINEL:
            self.extension_group_id = extension_group_id
        self.field = self._define_object(field, FieldSummary)
        if id_ is not SENTINEL:
            self.id_ = id_
        if process_id is not SENTINEL:
            self.process_id = process_id
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .account_cloud_attachment_summary_expression import AccountCloudAttachmentSummaryExpression, AccountCloudAttachmentSummaryExpressionGuard
from .account_cloud_attachment_summary_simple_expression import AccountCloudAttachmentSummarySimpleExpression
from .account_cloud_attachment_summary_grouping_expression import AccountCloudAttachmentSummaryGroupingExpression


@JsonMap({})
class AccountCloudAttachmentSummaryQueryConfigQueryFilter(BaseModel):
    """AccountCloudAttachmentSummaryQueryConfigQueryFilter

    :param expression: expression
    :type expression: AccountCloudAttachmentSummaryExpression
    """

    def __init__(self, expression: AccountCloudAttachmentSummaryExpression, **kwargs):
        """AccountCloudAttachmentSummaryQueryConfigQueryFilter

        :param expression: expression
        :type expression: AccountCloudAttachmentSummaryExpression
        """
        self.expression = AccountCloudAttachmentSummaryExpressionGuard.return_one_of(expression)
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class AccountCloudAttachmentSummaryQueryConfig(BaseModel):
    """AccountCloudAttachmentSummaryQueryConfig

    :param query_filter: query_filter
    :type query_filter: AccountCloudAttachmentSummaryQueryConfigQueryFilter
    """

    def __init__(self, query_filter: AccountCloudAttachmentSummaryQueryConfigQueryFilter, **kwargs):
        """AccountCloudAttachmentSummaryQueryConfig

        :param query_filter: query_filter
        :type query_filter: AccountCloudAttachmentSummaryQueryConfigQueryFilter
        """
        self.query_filter = self._define_object(
            query_filter, AccountCloudAttachmentSummaryQueryConfigQueryFilter
        )
        self._kwargs = kwargs




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap(
    {
        "bouncy_castle_library_version": "bouncyCastleLibraryVersion",
        "client_default_connect_timeout": "clientDefaultConnectTimeout",
        "client_default_read_timeout": "clientDefaultReadTimeout",
        "dns_time_to_live": "dnsTimeToLive",
        "endorsed_directories": "endorsedDirectories",
        "file_encoding": "fileEncoding",
        "headless_mode": "headlessMode",
        "heap_dump_on_out_of_memory_error": "heapDumpOnOutOfMemoryError",
        "heap_size": "heapSize",
        "java_class_path": "javaClassPath",
        "java_class_path_prepend": "javaClassPathPrepend",
        "java_library_path": "javaLibraryPath",
        "java_security_compatibility": "javaSecurityCompatibility",
        "jgroups_clustered_library_version": "jgroupsClusteredLibraryVersion",
        "jmx_remote_authentication": "jmxRemoteAuthentication",
        "jmx_remote_port": "jmxRemotePort",
        "jmx_remote_rmi_port": "jmxRemoteRmiPort",
        "jmx_remote_ssl": "jmxRemoteSsl",
        "non_proxy_hosts": "nonProxyHosts",
        "prefer_ipv4_stack": "preferIpv4Stack",
        "proxy_host": "proxyHost",
        "proxy_port": "proxyPort",
        "retry_http_post": "retryHttpPost",
        "temp_directory": "tempDirectory",
    }
)
class SystemProperties(BaseModel):
    """SystemProperties

    :param bouncy_castle_library_version: bouncy_castle_library_version, defaults to None
    :type bouncy_castle_library_version: str, optional
    :param client_default_connect_timeout: client_default_connect_timeout, defaults to None
    :type client_default_connect_timeout: int, optional
    :param client_default_read_timeout: client_default_read_timeout, defaults to None
    :type client_default_read_timeout: int, optional
    :param dns_time_to_live: dns_time_to_live, defaults to None
    :type dns_time_to_live: int, optional
    :param endorsed_directories: endorsed_directories, defaults to None
    :type endorsed_directories: str, optional
    :param file_encoding: file_encoding, defaults to None
    :type file_encoding: str, optional
    :param headless_mode: headless_mode, defaults to None
    :type headless_mode: bool, optional
    :param heap_dump_on_out_of_memory_error: heap_dump_on_out_of_memory_error, defaults to None
    :type heap_dump_on_out_of_memory_error: bool, optional
    :param heap_size: heap_size, defaults to None
    :type heap_size: str, optional
    :param java_class_path: java_class_path, defaults to None
    :type java_class_path: str, optional
    :param java_class_path_prepend: java_class_path_prepend, defaults to None
    :type java_class_path_prepend: str, optional
    :param java_library_path: java_library_path, defaults to None
    :type java_library_path: str, optional
    :param java_security_compatibility: java_security_compatibility, defaults to None
    :type java_security_compatibility: str, optional
    :param jgroups_clustered_library_version: jgroups_clustered_library_version, defaults to None
    :type jgroups_clustered_library_version: int, optional
    :param jmx_remote_authentication: jmx_remote_authentication, defaults to None
    :type jmx_remote_authentication: bool, optional
    :param jmx_remote_port: jmx_remote_port, defaults to None
    :type jmx_remote_port: int, optional
    :param jmx_remote_rmi_port: jmx_remote_rmi_port, defaults to None
    :type jmx_remote_rmi_port: int, optional
    :param jmx_remote_ssl: jmx_remote_ssl, defaults to None
    :type jmx_remote_ssl: bool, optional
    :param non_proxy_hosts: non_proxy_hosts, defaults to None
    :type non_proxy_hosts: str, optional
    :param prefer_ipv4_stack: prefer_ipv4_stack, defaults to None
    :type prefer_ipv4_stack: bool, optional
    :param proxy_host: proxy_host, defaults to None
    :type proxy_host: str, optional
    :param proxy_port: proxy_port, defaults to None
    :type proxy_port: str, optional
    :param retry_http_post: retry_http_post, defaults to None
    :type retry_http_post: bool, optional
    :param temp_directory: temp_directory, defaults to None
    :type temp_directory: str, optional
    """

    def __init__(
        self,
        bouncy_castle_library_version: str = SENTINEL,
        client_default_connect_timeout: int = SENTINEL,
        client_default_read_timeout: int = SENTINEL,
        dns_time_to_live: int = SENTINEL,
        endorsed_directories: str = SENTINEL,
        file_encoding: str = SENTINEL,
        headless_mode: bool = SENTINEL,
        heap_dump_on_out_of_memory_error: bool = SENTINEL,
        heap_size: str = SENTINEL,
        java_class_path: str = SENTINEL,
        java_class_path_prepend: str = SENTINEL,
        java_library_path: str = SENTINEL,
        java_security_compatibility: str = SENTINEL,
        jgroups_clustered_library_version: int = SENTINEL,
        jmx_remote_authentication: bool = SENTINEL,
        jmx_remote_port: int = SENTINEL,
        jmx_remote_rmi_port: int = SENTINEL,
        jmx_remote_ssl: bool = SENTINEL,
        non_proxy_hosts: str = SENTINEL,
        prefer_ipv4_stack: bool = SENTINEL,
        proxy_host: str = SENTINEL,
        proxy_port: str = SENTINEL,
        retry_http_post: bool = SENTINEL,
        temp_directory: str = SENTINEL,
        **kwargs,
    ):
        """SystemProperties - see class docstring for parameter details."""
        if bouncy_castle_library_version is not SENTINEL:
            self.bouncy_castle_library_version = bouncy_castle_library_version
        if client_default_connect_timeout is not SENTINEL:
            self.client_default_connect_timeout = client_default_connect_timeout
        if client_default_read_timeout is not SENTINEL:
            self.client_default_read_timeout = client_default_read_timeout
        if dns_time_to_live is not SENTINEL:
            self.dns_time_to_live = dns_time_to_live
        if endorsed_directories is not SENTINEL:
            self.endorsed_directories = endorsed_directories
        if file_encoding is not SENTINEL:
            self.file_encoding = file_encoding
        if headless_mode is not SENTINEL:
            self.headless_mode = headless_mode
        if heap_dump_on_out_of_memory_error is not SENTINEL:
            self.heap_dump_on_out_of_memory_error = heap_dump_on_out_of_memory_error
        if heap_size is not SENTINEL:
            self.heap_size = heap_size
        if java_class_path is not SENTINEL:
            self.java_class_path = java_class_path
        if java_class_path_prepend is not SENTINEL:
            self.java_class_path_prepend = java_class_path_prepend
        if java_library_path is not SENTINEL:
            self.java_library_path = java_library_path
        if java_security_compatibility is not SENTINEL:
            self.java_security_compatibility = java_security_compatibility
        if jgroups_clustered_library_version is not SENTINEL:
            self.jgroups_clustered_library_version = jgroups_clustered_library_version
        if jmx_remote_authentication is not SENTINEL:
            self.jmx_remote_authentication = jmx_remote_authentication
        if jmx_remote_port is not SENTINEL:
            self.jmx_remote_port = jmx_remote_port
        if jmx_remote_rmi_port is not SENTINEL:
            self.jmx_remote_rmi_port = jmx_remote_rmi_port
        if jmx_remote_ssl is not SENTINEL:
            self.jmx_remote_ssl = jmx_remote_ssl
        if non_proxy_hosts is not SENTINEL:
            self.non_proxy_hosts = non_proxy_hosts
        if prefer_ipv4_stack is not SENTINEL:
            self.prefer_ipv4_stack = prefer_ipv4_stack
        if proxy_host is not SENTINEL:
            self.proxy_host = proxy_host
        if proxy_port is not SENTINEL:
            self.proxy_port = proxy_port
        if retry_http_post is not SENTINEL:
            self.retry_http_post = retry_http_post
        if temp_directory is not SENTINEL:
            self.temp_directory = temp_directory
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .environment_map_extension_user_defined_function_summary_expression import (
    EnvironmentMapExtensionUserDefinedFunctionSummaryExpression,
    EnvironmentMapExtensionUserDefinedFunctionSummaryExpressionGuard,
)
from .environment_map_extension_user_defined_function_summary_simple_expression import (
    EnvironmentMapExtensionUserDefinedFunctionSummarySimpleExpression,
)
from .environment_map_extension_user_defined_function_summary_grouping_expression import (
    EnvironmentMapExtensionUserDefinedFunctionSummaryGroupingExpression,
)


@JsonMap({})
class EnvironmentMapExtensionUserDefinedFunctionSummaryQueryConfigQueryFilter(
    BaseModel
):
    """EnvironmentMapExtensionUserDefinedFunctionSummaryQueryConfigQueryFilter

    :param expression: expression
    :type expression: EnvironmentMapExtensionUserDefinedFunctionSummaryExpression
    """

    def __init__(
        self,
        expression: EnvironmentMapExtensionUserDefinedFunctionSummaryExpression,
        **kwargs,
    ):
        """EnvironmentMapExtensionUserDefinedFunctionSummaryQueryConfigQueryFilter

        :param expression: expression
        :type expression: EnvironmentMapExtensionUserDefinedFunctionSummaryExpression
        """
        self.expression = EnvironmentMapExtensionUserDefinedFunctionSummaryExpressionGuard.return_one_of(
            expression
        )
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class EnvironmentMapExtensionUserDefinedFunctionSummaryQueryConfig(BaseModel):
    """EnvironmentMapExtensionUserDefinedFunctionSummaryQueryConfig

    :param query_filter: query_filter
    :type query_filter: EnvironmentMapExtensionUserDefinedFunctionSummaryQueryConfigQueryFilter
    """

    def __init__(
        self,
        query_filter: EnvironmentMapExtensionUserDefinedFunctionSummaryQueryConfigQueryFilter,
        **kwargs,
    ):
        """EnvironmentMapExtensionUserDefinedFunctionSummaryQueryConfig

        :param query_filter: query_filter
        :type query_filter: EnvironmentMapExtensionUserDefinedFunctionSummaryQueryConfigQueryFilter
        """
        self.query_filter = self._define_object(
            query_filter,
            EnvironmentMapExtensionUserDefinedFunctionSummaryQueryConfigQueryFilter,
        )
        self._kwargs = kwargs




from enum import Enum
from typing import Any

# BaseModel is imported lazily inside extract_original_data to avoid
# importing the entire sdk.models package during module import which can
# introduce circular dependencies when only XML utilities are required.
try:
    import xmltodict
    ExpatError = xmltodict.expat.ExpatError
except Exception:  # pragma: no cover - if xmltodict is unavailable
    xmltodict = None
    from xml.parsers.expat import ExpatError  # type: ignore
    import xml.etree.ElementTree as _ET


def extract_original_data(data: Any) -> Any:
    """Extract the original data from internal models and enums.

    ``BaseModel`` is imported lazily to prevent importing the whole
    :mod:`sdk.models` package unless this function is actually used.

    :param Any data: The data to be extracted.
    :return: The extracted data.
    :rtype: Any
    """
    if data is None:
        return None

    data_type = type(data)

    try:
        from ...models.utils.base_model import BaseModel  # type: ignore
    except Exception:  # pragma: no cover - defensive fallback
        BaseModel = None

    if BaseModel is not None and issubclass(data_type, BaseModel):
        return data._map()

    if issubclass(data_type, Enum):
        return data.value

    if issubclass(data_type, list):
        return [extract_original_data(item) for item in data]

    return data


def _remove_namespaces(obj):
    """Recursively remove XML namespaces from dictionary keys.
    
    Converts keys like '{http://namespace}elementName' to 'elementName'.
    Handles nested dictionaries, lists, and preserves data structure.
    
    :param obj: The object to process (dict, list, or other)
    :return: Object with namespaces removed from dictionary keys
    """
    if isinstance(obj, dict):
        cleaned_dict = {}
        for key, value in obj.items():
            # Remove namespace from key: {namespace}name -> name
            if isinstance(key, str) and key.startswith('{') and '}' in key:
                # Extract the local name after the namespace
                clean_key = key.split('}', 1)[1]
            else:
                clean_key = key
            
            # Recursively clean the value
            cleaned_dict[clean_key] = _remove_namespaces(value)
        return cleaned_dict
    elif isinstance(obj, list):
        # Recursively clean list items
        return [_remove_namespaces(item) for item in obj]
    else:
        # Return primitive values unchanged
        return obj


def _extract_component_data(parsed_dict):
    """Extract API response data from root wrapper and normalize attribute keys.

    Handles API responses by:
    1. Extracting data from root elements (Component, QueryResult, PackagedComponent, etc.)
    2. Converting '@attributeName' keys to 'attributeName'
    3. Preserving nested structure for object, encryptedValues, result, etc.
    4. Handling empty xsi:type elements (empty async operation results)

    :param parsed_dict: Dictionary from XML parsing with namespaces removed
    :return: Data ready for JsonMap processing
    """
    # Check if this is an AsyncOperationResult response
    if 'AsyncOperationResult' in parsed_dict:
        async_result_data = parsed_dict['AsyncOperationResult']
        if isinstance(async_result_data, dict):
            normalized_data = _normalize_attribute_keys(async_result_data)

            # Special handling for empty result elements with xsi:type attribute
            # When <bns:result xsi:type="bns:SomeType"/> is empty, treat as None/empty
            if 'result' in normalized_data and isinstance(normalized_data['result'], dict):
                result_dict = normalized_data['result']
                # If result only contains a 'type' key, it's an empty element - treat as None
                if list(result_dict.keys()) == ['type']:
                    normalized_data['result'] = None

            # Ensure result is always a list (mirrors QueryResult behavior at lines 121-122).
            # xmltodict returns a single dict when there is one <result> element,
            # but *AsyncResponse models expect List[...].
            if 'result' in normalized_data and normalized_data['result'] is not None:
                if not isinstance(normalized_data['result'], list):
                    normalized_data['result'] = [normalized_data['result']]

            return normalized_data

    # Check if this is a Component response
    if 'Component' in parsed_dict:
        component_data = parsed_dict['Component']
        if isinstance(component_data, dict):
            return _normalize_attribute_keys(component_data)
    
    # Check if this is a QueryResult response (ExecutionRecord, etc.)
    elif 'QueryResult' in parsed_dict:
        query_result_data = parsed_dict['QueryResult']
        if isinstance(query_result_data, dict):
            normalized_data = _normalize_attribute_keys(query_result_data)

            # Special handling for QueryResult: ensure 'result' is always a list
            # The API returns a single dict when there's one result, but models expect a list
            if 'result' in normalized_data and not isinstance(normalized_data['result'], list):
                normalized_data['result'] = [normalized_data['result']]

            # Special handling for ProcessSchedules in QueryResult
            # Each ProcessSchedules result may have a Schedule field that needs to be a list
            if 'result' in normalized_data and isinstance(normalized_data['result'], list):
                for item in normalized_data['result']:
                    if isinstance(item, dict) and 'Schedule' in item:
                        if not isinstance(item['Schedule'], list):
                            item['Schedule'] = [item['Schedule']]

            return normalized_data
    
    # Check if this is a LogDownload response (ExecutionArtifacts)
    elif 'LogDownload' in parsed_dict:
        log_download_data = parsed_dict['LogDownload']
        if isinstance(log_download_data, dict):
            return _normalize_attribute_keys(log_download_data)

    # Check if this is a ProcessSchedules response
    elif 'ProcessSchedules' in parsed_dict:
        process_schedules_data = parsed_dict['ProcessSchedules']
        if isinstance(process_schedules_data, dict):
            normalized_data = _normalize_attribute_keys(process_schedules_data)

            # Special handling for ProcessSchedules: ensure 'Schedule' is always a list
            # The API returns a single dict when there's one schedule, but models expect a list
            if 'Schedule' in normalized_data and not isinstance(normalized_data['Schedule'], list):
                normalized_data['Schedule'] = [normalized_data['Schedule']]

            return normalized_data

    # Generic handling for other model types (PackagedComponent, Environment, etc.)
    # If the dict has a single key that looks like a model name (PascalCase),
    # extract its content and normalize
    if len(parsed_dict) == 1:
        root_key = list(parsed_dict.keys())[0]
        # Check if it looks like a model name (starts with uppercase)
        if root_key and root_key[0].isupper():
            inner_data = parsed_dict[root_key]
            if isinstance(inner_data, dict):
                return _normalize_attribute_keys(inner_data)
    
    # For other responses, return as-is
    return parsed_dict


def _normalize_attribute_keys(data_dict):
    """Normalize XML attribute keys for JsonMap compatibility.

    Converts '@attributeName' keys to 'attributeName' recursively.
    Also removes namespaces from attribute keys (e.g., '@{namespace}attr' -> 'attr').

    :param data_dict: Dictionary with potential XML attribute keys
    :return: Dictionary with normalized keys
    """
    if not isinstance(data_dict, dict):
        return data_dict

    normalized_data = {}

    for key, value in data_dict.items():
        # Convert @attributeName to attributeName for JsonMap compatibility
        if isinstance(key, str) and key.startswith('@'):
            clean_key = key[1:]  # Remove @ prefix
            # Also remove namespace if present: {namespace}name -> name
            if clean_key.startswith('{') and '}' in clean_key:
                clean_key = clean_key.split('}', 1)[1]
        else:
            clean_key = key

        # Recursively normalize nested dictionaries
        if isinstance(value, dict):
            normalized_data[clean_key] = _normalize_attribute_keys(value)
        elif isinstance(value, list):
            # Handle lists that might contain dictionaries
            normalized_data[clean_key] = [
                _normalize_attribute_keys(item) if isinstance(item, dict) else item
                for item in value
            ]
        else:
            normalized_data[clean_key] = value

    return normalized_data


def parse_xml_to_dict(xml_string: str) -> dict:
    """Parse an XML string into a dictionary.

    If :mod:`xmltodict` is available it will be used for the conversion with
    force_list parameter to handle duplicate XML elements correctly. When it
    is not installed, a fallback implementation based on
    :mod:`xml.etree.ElementTree` is used that properly handles duplicate elements.

    :param xml_string: The XML string to parse.
    :type xml_string: str
    :raises TypeError: If ``xml_string`` is not a string.
    :raises ExpatError: If the XML string is malformed.
    :return: A Python dictionary representing the XML structure.
    :rtype: dict
    """
    if not isinstance(xml_string, str):
        raise TypeError(
            f"Expected an XML string for parsing, but got type {type(xml_string).__name__}."
        )

    if xmltodict is not None:
        # Use force_list to ensure duplicate XML elements are converted to lists
        # Common Boomi API elements that may appear multiple times
        force_list_elements = [
            'shape', 'property', 'step', 'connection', 'component', 'item',
            'element', 'field', 'parameter', 'value', 'node', 'entry', 'Schedule'
        ]
        # Let xmltodict.parse raise its own errors for malformed XML.
        parsed = xmltodict.parse(xml_string, force_list=force_list_elements)
        
        # Remove namespaces and extract Component data for JsonMap compatibility
        cleaned = _remove_namespaces(parsed)
        return _extract_component_data(cleaned)

    # Enhanced fallback parser that handles duplicate elements correctly
    def _elem_to_dict(elem):
        children = list(elem)
        result = {f"@{k}": v for k, v in elem.attrib.items()}
        if children:
            child_dict = {}
            # Track element names to detect duplicates
            element_counts = {}
            
            for child in children:
                child_result = _elem_to_dict(child)
                child_tag = child.tag
                child_data = child_result[child_tag]
                
                # Count occurrences of this element name
                element_counts[child_tag] = element_counts.get(child_tag, 0) + 1
                
                if child_tag in child_dict:
                    # Convert to list if we encounter a duplicate
                    if not isinstance(child_dict[child_tag], list):
                        child_dict[child_tag] = [child_dict[child_tag]]
                    child_dict[child_tag].append(child_data)
                else:
                    child_dict[child_tag] = child_data
            
            if elem.text and elem.text.strip():
                result["#text"] = elem.text.strip()
            result = {elem.tag: {**result, **child_dict}}
        else:
            text = elem.text.strip() if elem.text and elem.text.strip() else None
            if result:
                if text is not None:
                    result["#text"] = text
                result = {elem.tag: result}
            else:
                result = {elem.tag: text}
        return result

    try:
        root = _ET.fromstring(xml_string)
    except _ET.ParseError as exc:  # pragma: no cover - matches xmltodict behaviour
        raise ExpatError(str(exc))
    
    # Parse with fallback parser and apply namespace/Component processing
    parsed = _elem_to_dict(root)
    cleaned = _remove_namespaces(parsed)
    return _extract_component_data(cleaned)


def parse_xml_to_dict_with_preservation(xml_string: str) -> dict:
    """Parse XML to dict while preserving original XML for round-tripping.
    
    This function works like parse_xml_to_dict but also stores the original
    XML and object element for perfect round-trip preservation. Used by
    Component model to enable lossless XML updates.
    
    :param xml_string: The XML string to parse
    :type xml_string: str
    :return: Dict with additional XML preservation fields
    :rtype: dict
    """
    from xml.etree import ElementTree as ET
    
    # First, do the normal parsing for backward compatibility
    result = parse_xml_to_dict(xml_string)
    
    # Now add XML preservation data
    try:
        root = ET.fromstring(xml_string)
        
        # Store the original XML
        result['_original_xml'] = xml_string
        
        # Extract and store the object element as raw XML
        ns = "http://api.platform.boomi.com/"
        obj_elem = root.find(f"{{{ns}}}object")
        if obj_elem is not None:
            # Store the inner XML (the content inside <bns:object>)
            inner_xml = ""
            for child in obj_elem:
                inner_xml += ET.tostring(child, encoding='unicode')
            result['object_xml'] = inner_xml
            result['_object_element'] = obj_elem
        
    except ET.ParseError:
        # If XML parsing fails, just use the regular result
        pass
    
    return result





from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap(
    {
        "archive_processed_documents": "archiveProcessedDocuments",
        "container_name": "containerName",
        "container_purge_days": "containerPurgeDays",
        "container_purge_immediately": "containerPurgeImmediately",
        "default_time_zone_for_account_schedules": "defaultTimeZoneForAccountSchedules",
        "low_latency_warning_threshold": "lowLatencyWarningThreshold",
        "partial_update": "partialUpdate",
        "purge_schedule_for_components": "purgeScheduleForComponents",
        "purge_schedule_for_logs": "purgeScheduleForLogs",
        "purge_schedule_for_processed_documents": "purgeScheduleForProcessedDocuments",
        "purge_schedule_for_temporary_data": "purgeScheduleForTemporaryData",
        "runtime_id": "runtimeId",
    }
)
class CloudAttachmentProperties(BaseModel):
    """CloudAttachmentProperties

    :param archive_processed_documents: archive_processed_documents, defaults to None
    :type archive_processed_documents: bool, optional
    :param container_name: container_name, defaults to None
    :type container_name: str, optional
    :param container_purge_days: container_purge_days, defaults to None
    :type container_purge_days: int, optional
    :param container_purge_immediately: container_purge_immediately, defaults to None
    :type container_purge_immediately: bool, optional
    :param default_time_zone_for_account_schedules: default_time_zone_for_account_schedules, defaults to None
    :type default_time_zone_for_account_schedules: str, optional
    :param low_latency_warning_threshold: low_latency_warning_threshold, defaults to None
    :type low_latency_warning_threshold: int, optional
    :param partial_update: partial_update, defaults to None
    :type partial_update: bool, optional
    :param purge_schedule_for_components: purge_schedule_for_components, defaults to None
    :type purge_schedule_for_components: int, optional
    :param purge_schedule_for_logs: purge_schedule_for_logs, defaults to None
    :type purge_schedule_for_logs: int, optional
    :param purge_schedule_for_processed_documents: purge_schedule_for_processed_documents, defaults to None
    :type purge_schedule_for_processed_documents: int, optional
    :param purge_schedule_for_temporary_data: purge_schedule_for_temporary_data, defaults to None
    :type purge_schedule_for_temporary_data: int, optional
    :param runtime_id: runtime_id, defaults to None
    :type runtime_id: str, optional
    """

    def __init__(
        self,
        archive_processed_documents: bool = SENTINEL,
        container_name: str = SENTINEL,
        container_purge_days: int = SENTINEL,
        container_purge_immediately: bool = SENTINEL,
        default_time_zone_for_account_schedules: str = SENTINEL,
        low_latency_warning_threshold: int = SENTINEL,
        partial_update: bool = SENTINEL,
        purge_schedule_for_components: int = SENTINEL,
        purge_schedule_for_logs: int = SENTINEL,
        purge_schedule_for_processed_documents: int = SENTINEL,
        purge_schedule_for_temporary_data: int = SENTINEL,
        runtime_id: str = SENTINEL,
        **kwargs,
    ):
        """CloudAttachmentProperties

        :param archive_processed_documents: archive_processed_documents, defaults to None
        :type archive_processed_documents: bool, optional
        :param container_name: container_name, defaults to None
        :type container_name: str, optional
        :param container_purge_days: container_purge_days, defaults to None
        :type container_purge_days: int, optional
        :param container_purge_immediately: container_purge_immediately, defaults to None
        :type container_purge_immediately: bool, optional
        :param default_time_zone_for_account_schedules: default_time_zone_for_account_schedules, defaults to None
        :type default_time_zone_for_account_schedules: str, optional
        :param low_latency_warning_threshold: low_latency_warning_threshold, defaults to None
        :type low_latency_warning_threshold: int, optional
        :param partial_update: partial_update, defaults to None
        :type partial_update: bool, optional
        :param purge_schedule_for_components: purge_schedule_for_components, defaults to None
        :type purge_schedule_for_components: int, optional
        :param purge_schedule_for_logs: purge_schedule_for_logs, defaults to None
        :type purge_schedule_for_logs: int, optional
        :param purge_schedule_for_processed_documents: purge_schedule_for_processed_documents, defaults to None
        :type purge_schedule_for_processed_documents: int, optional
        :param purge_schedule_for_temporary_data: purge_schedule_for_temporary_data, defaults to None
        :type purge_schedule_for_temporary_data: int, optional
        :param runtime_id: runtime_id, defaults to None
        :type runtime_id: str, optional
        """
        if archive_processed_documents is not SENTINEL:
            self.archive_processed_documents = archive_processed_documents
        if container_name is not SENTINEL:
            self.container_name = container_name
        if container_purge_days is not SENTINEL:
            self.container_purge_days = container_purge_days
        if container_purge_immediately is not SENTINEL:
            self.container_purge_immediately = container_purge_immediately
        if default_time_zone_for_account_schedules is not SENTINEL:
            self.default_time_zone_for_account_schedules = (
                default_time_zone_for_account_schedules
            )
        if low_latency_warning_threshold is not SENTINEL:
            self.low_latency_warning_threshold = low_latency_warning_threshold
        if partial_update is not SENTINEL:
            self.partial_update = partial_update
        if purge_schedule_for_components is not SENTINEL:
            self.purge_schedule_for_components = purge_schedule_for_components
        if purge_schedule_for_logs is not SENTINEL:
            self.purge_schedule_for_logs = purge_schedule_for_logs
        if purge_schedule_for_processed_documents is not SENTINEL:
            self.purge_schedule_for_processed_documents = (
                purge_schedule_for_processed_documents
            )
        if purge_schedule_for_temporary_data is not SENTINEL:
            self.purge_schedule_for_temporary_data = purge_schedule_for_temporary_data
        if runtime_id is not SENTINEL:
            self.runtime_id = runtime_id
        self._kwargs = kwargs




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..move_queue_request import MoveQueueRequestService
from ...models import MoveQueueRequest


class MoveQueueRequestServiceAsync(MoveQueueRequestService):
    """
    Async Wrapper for MoveQueueRequestServiceAsync
    """

    def create_move_queue_request(
        self, request_body: MoveQueueRequest = None
    ) -> Awaitable[Union[MoveQueueRequest, str]]:
        return to_async(super().create_move_queue_request)(request_body)




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap({"raw_size": "rawSize"})
class AtomDiskSpaceDirectory(BaseModel):
    """AtomDiskSpaceDirectory

    :param file: A specific directory within the attachment’s directory for which this size pertains., defaults to None
    :type file: str, optional
    :param raw_size: The disk space in bytes that is already consumed within the directory., defaults to None
    :type raw_size: int, optional
    :param size: The size of disk space consumed within this directory., defaults to None
    :type size: str, optional
    """

    def __init__(
        self,
        file: str = SENTINEL,
        raw_size: int = SENTINEL,
        size: str = SENTINEL,
        **kwargs
    ):
        """AtomDiskSpaceDirectory

        :param file: A specific directory within the attachment’s directory for which this size pertains., defaults to None
        :type file: str, optional
        :param raw_size: The disk space in bytes that is already consumed within the directory., defaults to None
        :type raw_size: int, optional
        :param size: The size of disk space consumed within this directory., defaults to None
        :type size: str, optional
        """
        if file is not SENTINEL:
            self.file = file
        if raw_size is not SENTINEL:
            self.raw_size = raw_size
        if size is not SENTINEL:
            self.size = size
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .observability_endpoint import ObservabilityEndpoint


@JsonMap(
    {
        "metric_interval_seconds": "metricIntervalSeconds",
        "metric_timeout_seconds": "metricTimeoutSeconds",
        "observability_endpoint": "observabilityEndpoint",
        "override_general_observability_endpoint": "overrideGeneralObservabilityEndpoint",
    }
)
class MetricSettings(BaseModel):
    """MetricSettings

    :param metric_interval_seconds: metric_interval_seconds
    :type metric_interval_seconds: int
    :param metric_timeout_seconds: metric_timeout_seconds
    :type metric_timeout_seconds: int
    :param observability_endpoint: observability_endpoint
    :type observability_endpoint: ObservabilityEndpoint
    :param enabled: enabled, defaults to None
    :type enabled: bool, optional
    :param override_general_observability_endpoint: override_general_observability_endpoint, defaults to None
    :type override_general_observability_endpoint: bool, optional
    """

    def __init__(
        self,
        metric_interval_seconds: int,
        metric_timeout_seconds: int,
        observability_endpoint: ObservabilityEndpoint,
        enabled: bool = SENTINEL,
        override_general_observability_endpoint: bool = SENTINEL,
        **kwargs,
    ):
        """MetricSettings

        :param metric_interval_seconds: metric_interval_seconds
        :type metric_interval_seconds: int
        :param metric_timeout_seconds: metric_timeout_seconds
        :type metric_timeout_seconds: int
        :param observability_endpoint: observability_endpoint
        :type observability_endpoint: ObservabilityEndpoint
        :param enabled: enabled, defaults to None
        :type enabled: bool, optional
        :param override_general_observability_endpoint: override_general_observability_endpoint, defaults to None
        :type override_general_observability_endpoint: bool, optional
        """
        self.metric_interval_seconds = metric_interval_seconds
        self.metric_timeout_seconds = metric_timeout_seconds
        self.observability_endpoint = self._define_object(
            observability_endpoint, ObservabilityEndpoint
        )
        if enabled is not SENTINEL:
            self.enabled = enabled
        if override_general_observability_endpoint is not SENTINEL:
            self.override_general_observability_endpoint = (
                override_general_observability_endpoint
            )
        self._kwargs = kwargs




from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..net.transport.utils import parse_xml_to_dict
from ..models import ClearQueueRequest


class ClearQueueService(BaseService):

    @cast_models
    def execute_clear_queue(
        self, id_: str, request_body: ClearQueueRequest = None
    ) -> Union[ClearQueueRequest, str]:
        """- When you run the Clear queue messages action, it deletes all messages in a queue name from the queue. Note that this clears all messages in the queue; you cannot select and remove individual messages using this action. In addition, the action overrides any purge settings you might configure in the user interface.
         - The immediate response indicates success in passing the request to the Runtime.
         - If the specified Runtime queue does not contain any messages to clear, the response only returns a success message stating that the message passed even though there is no action taken on the Runtime.
         - You can use the Get queue list API action to retrieve the number of messages in a queue, which works as an alternative way to check if the clear queue message action succeeded on the Runtime.

        :param request_body: The request body., defaults to None
        :type request_body: ClearQueueRequest, optional
        :param id_: The unique ID assigned by the system to the container. The Runtime ID for Runtimes, Runtime clusters, and Runtime clouds is found in the user interface by navigating to Manage > Runtime Management and viewing the Runtime Information panel for the selected container.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[ClearQueueRequest, str]
        """

        Validator(ClearQueueRequest).is_optional().validate(request_body)
        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/ClearQueue/execute/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return ClearQueueRequest._unmap(response)
        if content == "application/xml":
            return ClearQueueRequest._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..runtime_cloud import RuntimeCloudService
from ...models import (
    RuntimeCloud,
    RuntimeCloudBulkResponse,
    RuntimeCloudBulkRequest,
    RuntimeCloudQueryResponse,
    RuntimeCloudQueryConfig,
)


class RuntimeCloudServiceAsync(RuntimeCloudService):
    """
    Async Wrapper for RuntimeCloudServiceAsync
    """

    def create_runtime_cloud(
        self, request_body: RuntimeCloud = None
    ) -> Awaitable[Union[RuntimeCloud, str]]:
        return to_async(super().create_runtime_cloud)(request_body)

    def get_runtime_cloud(self, id_: str) -> Awaitable[Union[RuntimeCloud, str]]:
        return to_async(super().get_runtime_cloud)(id_)

    def update_runtime_cloud(
        self, id_: str, request_body: RuntimeCloud = None
    ) -> Awaitable[Union[RuntimeCloud, str]]:
        return to_async(super().update_runtime_cloud)(id_, request_body)

    def delete_runtime_cloud(self, id_: str) -> Awaitable[None]:
        return to_async(super().delete_runtime_cloud)(id_)

    def bulk_runtime_cloud(
        self, request_body: RuntimeCloudBulkRequest = None
    ) -> Awaitable[Union[RuntimeCloudBulkResponse, str]]:
        return to_async(super().bulk_runtime_cloud)(request_body)

    def query_runtime_cloud(
        self, request_body: RuntimeCloudQueryConfig = None
    ) -> Awaitable[Union[RuntimeCloudQueryResponse, str]]:
        return to_async(super().query_runtime_cloud)(request_body)

    def query_more_runtime_cloud(
        self, request_body: str
    ) -> Awaitable[Union[RuntimeCloudQueryResponse, str]]:
        return to_async(super().query_more_runtime_cloud)(request_body)




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap(
    {
        "account_id": "accountId",
        "certificate_id": "certificateId",
        "certificate_name": "certificateName",
        "certificate_type": "certificateType",
        "container_id": "containerId",
        "container_name": "containerName",
        "environment_id": "environmentId",
        "environment_name": "environmentName",
        "expiration_date": "expirationDate",
        "expiration_boundary": "expirationBoundary",
    }
)
class DeployedExpiredCertificate(BaseModel):
    """DeployedExpiredCertificate

    :param account_id: The ID of the account in which you deployed the certificate., defaults to None
    :type account_id: str, optional
    :param certificate_id: The ID of the certificate., defaults to None
    :type certificate_id: str, optional
    :param certificate_name: The name of the certificate., defaults to None
    :type certificate_name: str, optional
    :param certificate_type: The type of the certificate — X.509., defaults to None
    :type certificate_type: str, optional
    :param container_id: The ID of the container \(Runtime, Runtime cluster, or Runtime cloud\) to which you deployed the certificate., defaults to None
    :type container_id: str, optional
    :param container_name: The name of the container \(Runtime, Runtime cluster, or Runtime cloud\) to which you deployed the certificate., defaults to None
    :type container_name: str, optional
    :param environment_id: If applicable, the ID of the environment in which you deployed the certificate., defaults to None
    :type environment_id: str, optional
    :param environment_name: If applicable, the environment in which you deployed the certificate., defaults to None
    :type environment_name: str, optional
    :param expiration_date: The expiration date and time of the certificate., defaults to None
    :type expiration_date: str, optional
    :param location: The location to which you deployed the certificate — either Process, Environment, Runtime, Shared Web Server, Shared Web Server Authorization, or Shared Web Server User Authorization. Environment is valid only for an environment account; Runtime is valid only for a non-environment account., defaults to None
    :type location: str, optional
    :param expiration_boundary: sets the boundary for a timespan filter beginning or ending with the current date. The value is the number of days relative to the current date.   For example, use a value of 1 to retrieve certificates that expire today or tomorrow. Use a value of 365 to retrieve certificates that expire within the next 365 days. Use a value of –1 to retrieve certificates that expired yesterday or expire today., defaults to None
    :type expiration_boundary: int, optional
    """

    def __init__(
        self,
        account_id: str = SENTINEL,
        certificate_id: str = SENTINEL,
        certificate_name: str = SENTINEL,
        certificate_type: str = SENTINEL,
        container_id: str = SENTINEL,
        container_name: str = SENTINEL,
        environment_id: str = SENTINEL,
        environment_name: str = SENTINEL,
        expiration_date: str = SENTINEL,
        location: str = SENTINEL,
        expiration_boundary: int = SENTINEL,
        **kwargs
    ):
        """DeployedExpiredCertificate

        :param account_id: The ID of the account in which you deployed the certificate., defaults to None
        :type account_id: str, optional
        :param certificate_id: The ID of the certificate., defaults to None
        :type certificate_id: str, optional
        :param certificate_name: The name of the certificate., defaults to None
        :type certificate_name: str, optional
        :param certificate_type: The type of the certificate — X.509., defaults to None
        :type certificate_type: str, optional
        :param container_id: The ID of the container \(Runtime, Runtime cluster, or Runtime cloud\) to which you deployed the certificate., defaults to None
        :type container_id: str, optional
        :param container_name: The name of the container \(Runtime, Runtime cluster, or Runtime cloud\) to which you deployed the certificate., defaults to None
        :type container_name: str, optional
        :param environment_id: If applicable, the ID of the environment in which you deployed the certificate., defaults to None
        :type environment_id: str, optional
        :param environment_name: If applicable, the environment in which you deployed the certificate., defaults to None
        :type environment_name: str, optional
        :param expiration_date: The expiration date and time of the certificate., defaults to None
        :type expiration_date: str, optional
        :param location: The location to which you deployed the certificate — either Process, Environment, Runtime, Shared Web Server, Shared Web Server Authorization, or Shared Web Server User Authorization. Environment is valid only for an environment account; Runtime is valid only for a non-environment account., defaults to None
        :type location: str, optional
        :param expiration_boundary: sets the boundary for a timespan filter beginning or ending with the current date. The value is the number of days relative to the current date.   For example, use a value of 1 to retrieve certificates that expire today or tomorrow. Use a value of 365 to retrieve certificates that expire within the next 365 days. Use a value of –1 to retrieve certificates that expired yesterday or expire today., defaults to None
        :type expiration_boundary: int, optional
        """
        if account_id is not SENTINEL:
            self.account_id = account_id
        if certificate_id is not SENTINEL:
            self.certificate_id = certificate_id
        if certificate_name is not SENTINEL:
            self.certificate_name = certificate_name
        if certificate_type is not SENTINEL:
            self.certificate_type = certificate_type
        if container_id is not SENTINEL:
            self.container_id = container_id
        if container_name is not SENTINEL:
            self.container_name = container_name
        if environment_id is not SENTINEL:
            self.environment_id = environment_id
        if environment_name is not SENTINEL:
            self.environment_name = environment_name
        if expiration_date is not SENTINEL:
            self.expiration_date = expiration_date
        if location is not SENTINEL:
            self.location = location
        if expiration_boundary is not SENTINEL:
            self.expiration_boundary = expiration_boundary
        self._kwargs = kwargs




from typing import Union
from .net.environment import Environment
from .sdk import Boomi
from .services.async_.as2_connector_record import As2ConnectorRecordServiceAsync
from .services.async_.account import AccountServiceAsync
from .services.async_.account_cloud_attachment_properties import (
    AccountCloudAttachmentPropertiesServiceAsync,
)
from .services.async_.account_cloud_attachment_quota import (
    AccountCloudAttachmentQuotaServiceAsync,
)
from .services.async_.account_group import AccountGroupServiceAsync
from .services.async_.account_group_account import AccountGroupAccountServiceAsync
from .services.async_.account_group_user_role import AccountGroupUserRoleServiceAsync
from .services.async_.account_sso_config import AccountSsoConfigServiceAsync
from .services.async_.account_user_federation import AccountUserFederationServiceAsync
from .services.async_.account_user_role import AccountUserRoleServiceAsync
from .services.async_.api_usage_count import ApiUsageCountServiceAsync
from .services.async_.atom import AtomServiceAsync
from .services.async_.atom_as2_artifacts import AtomAs2ArtifactsServiceAsync
from .services.async_.atom_connection_field_extension_summary import (
    AtomConnectionFieldExtensionSummaryServiceAsync,
)
from .services.async_.atom_connector_versions import AtomConnectorVersionsServiceAsync
from .services.async_.atom_counters import AtomCountersServiceAsync
from .services.async_.atom_log import AtomLogServiceAsync
from .services.async_.atom_purge import AtomPurgeServiceAsync
from .services.async_.atom_security_policies import AtomSecurityPoliciesServiceAsync
from .services.async_.atom_startup_properties import AtomStartupPropertiesServiceAsync
from .services.async_.atom_worker_log import AtomWorkerLogServiceAsync
from .services.async_.audit_log import AuditLogServiceAsync
from .services.async_.branch import BranchServiceAsync
from .services.async_.change_listener_status import ChangeListenerStatusServiceAsync
from .services.async_.clear_queue import ClearQueueServiceAsync
from .services.async_.cloud import CloudServiceAsync
from .services.async_.cloud_attachment_properties import (
    CloudAttachmentPropertiesServiceAsync,
)
from .services.async_.component import ComponentServiceAsync
from .services.async_.component_atom_attachment import (
    ComponentAtomAttachmentServiceAsync,
)
from .services.async_.component_diff_request import ComponentDiffRequestServiceAsync
from .services.async_.component_environment_attachment import (
    ComponentEnvironmentAttachmentServiceAsync,
)
from .services.async_.component_metadata import ComponentMetadataServiceAsync
from .services.async_.component_reference import ComponentReferenceServiceAsync
from .services.async_.connection_licensing_report import (
    ConnectionLicensingReportServiceAsync,
)
from .services.async_.connector import ConnectorServiceAsync
from .services.async_.connector_document import ConnectorDocumentServiceAsync
from .services.async_.custom_tracked_field import CustomTrackedFieldServiceAsync
from .services.async_.deployed_expired_certificate import (
    DeployedExpiredCertificateServiceAsync,
)
from .services.async_.deployed_package import DeployedPackageServiceAsync
from .services.async_.deployment import DeploymentServiceAsync
from .services.async_.document_count_account import DocumentCountAccountServiceAsync
from .services.async_.document_count_account_group import (
    DocumentCountAccountGroupServiceAsync,
)
from .services.async_.edifact_connector_record import EdifactConnectorRecordServiceAsync
from .services.async_.edi_custom_connector_record import (
    EdiCustomConnectorRecordServiceAsync,
)
from .services.async_.environment import EnvironmentServiceAsync
from .services.async_.environment_atom_attachment import (
    EnvironmentAtomAttachmentServiceAsync,
)
from .services.async_.environment_connection_field_extension_summary import (
    EnvironmentConnectionFieldExtensionSummaryServiceAsync,
)
from .services.async_.environment_extensions import EnvironmentExtensionsServiceAsync
from .services.async_.environment_map_extension import (
    EnvironmentMapExtensionServiceAsync,
)
from .services.async_.environment_map_extension_external_component import (
    EnvironmentMapExtensionExternalComponentServiceAsync,
)
from .services.async_.environment_map_extension_user_defined_function import (
    EnvironmentMapExtensionUserDefinedFunctionServiceAsync,
)
from .services.async_.environment_map_extension_user_defined_function_summary import (
    EnvironmentMapExtensionUserDefinedFunctionSummaryServiceAsync,
)
from .services.async_.environment_map_extensions_summary import (
    EnvironmentMapExtensionsSummaryServiceAsync,
)
from .services.async_.environment_role import EnvironmentRoleServiceAsync
from .services.async_.event import EventServiceAsync
from .services.async_.execution_artifacts import ExecutionArtifactsServiceAsync
from .services.async_.execution_connector import ExecutionConnectorServiceAsync
from .services.async_.execution_count_account import ExecutionCountAccountServiceAsync
from .services.async_.execution_count_account_group import (
    ExecutionCountAccountGroupServiceAsync,
)
from .services.async_.execution_record import ExecutionRecordServiceAsync
from .services.async_.execution_request import ExecutionRequestServiceAsync
from .services.async_.execution_summary_record import ExecutionSummaryRecordServiceAsync
from .services.async_.folder import FolderServiceAsync
from .services.async_.generic_connector_record import GenericConnectorRecordServiceAsync
from .services.async_.get_assignable_roles import GetAssignableRolesServiceAsync
from .services.async_.hl7_connector_record import Hl7ConnectorRecordServiceAsync
from .services.async_.installer_token import InstallerTokenServiceAsync
from .services.async_.integration_pack import IntegrationPackServiceAsync
from .services.async_.integration_pack_atom_attachment import (
    IntegrationPackAtomAttachmentServiceAsync,
)
from .services.async_.integration_pack_environment_attachment import (
    IntegrationPackEnvironmentAttachmentServiceAsync,
)
from .services.async_.integration_pack_instance import (
    IntegrationPackInstanceServiceAsync,
)
from .services.async_.java_rollback import JavaRollbackServiceAsync
from .services.async_.java_upgrade import JavaUpgradeServiceAsync
from .services.async_.merge_request import MergeRequestServiceAsync
from .services.async_.move_queue_request import MoveQueueRequestServiceAsync
from .services.async_.node_offboard import NodeOffboardServiceAsync
from .services.async_.odette_connector_record import OdetteConnectorRecordServiceAsync
from .services.async_.oftp2_connector_record import Oftp2ConnectorRecordServiceAsync
from .services.async_.packaged_component import PackagedComponentServiceAsync
from .services.async_.packaged_component_manifest import (
    PackagedComponentManifestServiceAsync,
)
from .services.async_.persisted_process_properties import (
    PersistedProcessPropertiesServiceAsync,
)
from .services.async_.process import ProcessServiceAsync
from .services.async_.process_atom_attachment import ProcessAtomAttachmentServiceAsync
from .services.async_.process_environment_attachment import (
    ProcessEnvironmentAttachmentServiceAsync,
)
from .services.async_.process_log import ProcessLogServiceAsync
from .services.async_.process_schedule_status import ProcessScheduleStatusServiceAsync
from .services.async_.process_schedules import ProcessSchedulesServiceAsync
from .services.async_.rerun_document import RerunDocumentServiceAsync
from .services.async_.role import RoleServiceAsync
from .services.async_.rosetta_net_connector_record import (
    RosettaNetConnectorRecordServiceAsync,
)
from .services.async_.runtime_release_schedule import RuntimeReleaseScheduleServiceAsync
from .services.async_.cancel_execution import CancelExecutionServiceAsync
from .services.async_.runtime_cloud import RuntimeCloudServiceAsync
from .services.async_.account_cloud_attachment_summary import (
    AccountCloudAttachmentSummaryServiceAsync,
)
from .services.async_.account_cloud_attachment_properties_default import (
    AccountCloudAttachmentPropertiesDefaultServiceAsync,
)
from .services.async_.runtime_properties import RuntimePropertiesServiceAsync
from .services.async_.runtime_observability_settings import (
    RuntimeObservabilitySettingsServiceAsync,
)
from .services.async_.cloud_attachment_secrets_configuration import (
    CloudAttachmentSecretsConfigurationServiceAsync,
)
from .services.async_.shared_server_information import (
    SharedServerInformationServiceAsync,
)
from .services.async_.shared_web_server import SharedWebServerServiceAsync
from .services.async_.throughput_account import ThroughputAccountServiceAsync
from .services.async_.throughput_account_group import ThroughputAccountGroupServiceAsync
from .services.async_.tradacoms_connector_record import (
    TradacomsConnectorRecordServiceAsync,
)
from .services.async_.trading_partner_component import (
    TradingPartnerComponentServiceAsync,
)
from .services.async_.trading_partner_processing_group import (
    TradingPartnerProcessingGroupServiceAsync,
)
from .services.async_.x12_connector_record import X12ConnectorRecordServiceAsync
from .services.async_.atom_disk_space import AtomDiskSpaceServiceAsync
from .services.async_.list_queues import ListQueuesServiceAsync
from .services.async_.listener_status import ListenerStatusServiceAsync
from .services.async_.organization_component import OrganizationComponentServiceAsync
from .services.async_.shared_communication_channel_component import (
    SharedCommunicationChannelComponentServiceAsync,
)
from .services.async_.account_group_integration_pack import (
    AccountGroupIntegrationPackServiceAsync,
)
from .services.async_.publisher_integration_pack import (
    PublisherIntegrationPackServiceAsync,
)
from .services.async_.release_integration_pack import ReleaseIntegrationPackServiceAsync
from .services.async_.release_integration_pack_status import (
    ReleaseIntegrationPackStatusServiceAsync,
)
from .services.async_.runtime_restart_request import RuntimeRestartRequestServiceAsync
from .services.async_.refresh_secrets_manager import RefreshSecretsManagerServiceAsync


class BoomiAsync(Boomi):
    """
    BoomiAsync is the asynchronous version of the Boomi SDK Client.
    """

    def __init__(
        self,
        access_token: str = None,
        username: str = None,
        password: str = None,
        base_url: Union[Environment, str, None] = None,
        timeout: int = 60000,
        account_id: str = "platform_account_ID",
    ):
        super().__init__(
            access_token=access_token,
            username=username,
            password=password,
            base_url=base_url,
            timeout=timeout,
            account_id=account_id,
        )

        self.as2_connector_record = As2ConnectorRecordServiceAsync(
            base_url=self._base_url
        )
        self.account = AccountServiceAsync(base_url=self._base_url)
        self.account_cloud_attachment_properties = (
            AccountCloudAttachmentPropertiesServiceAsync(base_url=self._base_url)
        )
        self.account_cloud_attachment_quota = AccountCloudAttachmentQuotaServiceAsync(
            base_url=self._base_url
        )
        self.account_group = AccountGroupServiceAsync(base_url=self._base_url)
        self.account_group_account = AccountGroupAccountServiceAsync(
            base_url=self._base_url
        )
        self.account_group_user_role = AccountGroupUserRoleServiceAsync(
            base_url=self._base_url
        )
        self.account_sso_config = AccountSsoConfigServiceAsync(base_url=self._base_url)
        self.account_user_federation = AccountUserFederationServiceAsync(
            base_url=self._base_url
        )
        self.account_user_role = AccountUserRoleServiceAsync(base_url=self._base_url)
        self.api_usage_count = ApiUsageCountServiceAsync(base_url=self._base_url)
        self.atom = AtomServiceAsync(base_url=self._base_url)
        self.atom_as2_artifacts = AtomAs2ArtifactsServiceAsync(base_url=self._base_url)
        self.atom_connection_field_extension_summary = (
            AtomConnectionFieldExtensionSummaryServiceAsync(base_url=self._base_url)
        )
        self.atom_connector_versions = AtomConnectorVersionsServiceAsync(
            base_url=self._base_url
        )
        self.atom_counters = AtomCountersServiceAsync(base_url=self._base_url)
        self.atom_log = AtomLogServiceAsync(base_url=self._base_url)
        self.atom_purge = AtomPurgeServiceAsync(base_url=self._base_url)
        self.atom_security_policies = AtomSecurityPoliciesServiceAsync(
            base_url=self._base_url
        )
        self.atom_startup_properties = AtomStartupPropertiesServiceAsync(
            base_url=self._base_url
        )
        self.atom_worker_log = AtomWorkerLogServiceAsync(base_url=self._base_url)
        self.audit_log = AuditLogServiceAsync(base_url=self._base_url)
        self.branch = BranchServiceAsync(base_url=self._base_url)
        self.change_listener_status = ChangeListenerStatusServiceAsync(
            base_url=self._base_url
        )
        self.clear_queue = ClearQueueServiceAsync(base_url=self._base_url)
        self.cloud = CloudServiceAsync(base_url=self._base_url)
        self.cloud_attachment_properties = CloudAttachmentPropertiesServiceAsync(
            base_url=self._base_url
        )
        self.component = ComponentServiceAsync(base_url=self._base_url)
        self.component_atom_attachment = ComponentAtomAttachmentServiceAsync(
            base_url=self._base_url
        )
        self.component_diff_request = ComponentDiffRequestServiceAsync(
            base_url=self._base_url
        )
        self.component_environment_attachment = (
            ComponentEnvironmentAttachmentServiceAsync(base_url=self._base_url)
        )
        self.component_metadata = ComponentMetadataServiceAsync(base_url=self._base_url)
        self.component_reference = ComponentReferenceServiceAsync(
            base_url=self._base_url
        )
        self.connection_licensing_report = ConnectionLicensingReportServiceAsync(
            base_url=self._base_url
        )
        self.connector = ConnectorServiceAsync(base_url=self._base_url)
        self.connector_document = ConnectorDocumentServiceAsync(base_url=self._base_url)
        self.custom_tracked_field = CustomTrackedFieldServiceAsync(
            base_url=self._base_url
        )
        self.deployed_expired_certificate = DeployedExpiredCertificateServiceAsync(
            base_url=self._base_url
        )
        self.deployed_package = DeployedPackageServiceAsync(base_url=self._base_url)
        self.deployment = DeploymentServiceAsync(base_url=self._base_url)
        self.document_count_account = DocumentCountAccountServiceAsync(
            base_url=self._base_url
        )
        self.document_count_account_group = DocumentCountAccountGroupServiceAsync(
            base_url=self._base_url
        )
        self.edifact_connector_record = EdifactConnectorRecordServiceAsync(
            base_url=self._base_url
        )
        self.edi_custom_connector_record = EdiCustomConnectorRecordServiceAsync(
            base_url=self._base_url
        )
        self.environment = EnvironmentServiceAsync(base_url=self._base_url)
        self.environment_atom_attachment = EnvironmentAtomAttachmentServiceAsync(
            base_url=self._base_url
        )
        self.environment_connection_field_extension_summary = (
            EnvironmentConnectionFieldExtensionSummaryServiceAsync(
                base_url=self._base_url
            )
        )
        self.environment_extensions = EnvironmentExtensionsServiceAsync(
            base_url=self._base_url
        )
        self.environment_map_extension = EnvironmentMapExtensionServiceAsync(
            base_url=self._base_url
        )
        self.environment_map_extension_external_component = (
            EnvironmentMapExtensionExternalComponentServiceAsync(
                base_url=self._base_url
            )
        )
        self.environment_map_extension_user_defined_function = (
            EnvironmentMapExtensionUserDefinedFunctionServiceAsync(
                base_url=self._base_url
            )
        )
        self.environment_map_extension_user_defined_function_summary = (
            EnvironmentMapExtensionUserDefinedFunctionSummaryServiceAsync(
                base_url=self._base_url
            )
        )
        self.environment_map_extensions_summary = (
            EnvironmentMapExtensionsSummaryServiceAsync(base_url=self._base_url)
        )
        self.environment_role = EnvironmentRoleServiceAsync(base_url=self._base_url)
        self.event = EventServiceAsync(base_url=self._base_url)
        self.execution_artifacts = ExecutionArtifactsServiceAsync(
            base_url=self._base_url
        )
        self.execution_connector = ExecutionConnectorServiceAsync(
            base_url=self._base_url
        )
        self.execution_count_account = ExecutionCountAccountServiceAsync(
            base_url=self._base_url
        )
        self.execution_count_account_group = ExecutionCountAccountGroupServiceAsync(
            base_url=self._base_url
        )
        self.execution_record = ExecutionRecordServiceAsync(base_url=self._base_url)
        self.execution_request = ExecutionRequestServiceAsync(base_url=self._base_url)
        self.execution_summary_record = ExecutionSummaryRecordServiceAsync(
            base_url=self._base_url
        )
        self.folder = FolderServiceAsync(base_url=self._base_url)
        self.generic_connector_record = GenericConnectorRecordServiceAsync(
            base_url=self._base_url
        )
        self.get_assignable_roles = GetAssignableRolesServiceAsync(
            base_url=self._base_url
        )
        self.hl7_connector_record = Hl7ConnectorRecordServiceAsync(
            base_url=self._base_url
        )
        self.installer_token = InstallerTokenServiceAsync(base_url=self._base_url)
        self.integration_pack = IntegrationPackServiceAsync(base_url=self._base_url)
        self.integration_pack_atom_attachment = (
            IntegrationPackAtomAttachmentServiceAsync(base_url=self._base_url)
        )
        self.integration_pack_environment_attachment = (
            IntegrationPackEnvironmentAttachmentServiceAsync(base_url=self._base_url)
        )
        self.integration_pack_instance = IntegrationPackInstanceServiceAsync(
            base_url=self._base_url
        )
        self.java_rollback = JavaRollbackServiceAsync(base_url=self._base_url)
        self.java_upgrade = JavaUpgradeServiceAsync(base_url=self._base_url)
        self.merge_request = MergeRequestServiceAsync(base_url=self._base_url)
        self.move_queue_request = MoveQueueRequestServiceAsync(base_url=self._base_url)
        self.node_offboard = NodeOffboardServiceAsync(base_url=self._base_url)
        self.odette_connector_record = OdetteConnectorRecordServiceAsync(
            base_url=self._base_url
        )
        self.oftp2_connector_record = Oftp2ConnectorRecordServiceAsync(
            base_url=self._base_url
        )
        self.packaged_component = PackagedComponentServiceAsync(base_url=self._base_url)
        self.packaged_component_manifest = PackagedComponentManifestServiceAsync(
            base_url=self._base_url
        )
        self.persisted_process_properties = PersistedProcessPropertiesServiceAsync(
            base_url=self._base_url
        )
        self.process = ProcessServiceAsync(base_url=self._base_url)
        self.process_atom_attachment = ProcessAtomAttachmentServiceAsync(
            base_url=self._base_url
        )
        self.process_environment_attachment = ProcessEnvironmentAttachmentServiceAsync(
            base_url=self._base_url
        )
        self.process_log = ProcessLogServiceAsync(base_url=self._base_url)
        self.process_schedule_status = ProcessScheduleStatusServiceAsync(
            base_url=self._base_url
        )
        self.process_schedules = ProcessSchedulesServiceAsync(base_url=self._base_url)
        self.rerun_document = RerunDocumentServiceAsync(base_url=self._base_url)
        self.role = RoleServiceAsync(base_url=self._base_url)
        self.rosetta_net_connector_record = RosettaNetConnectorRecordServiceAsync(
            base_url=self._base_url
        )
        self.runtime_release_schedule = RuntimeReleaseScheduleServiceAsync(
            base_url=self._base_url
        )
        self.cancel_execution = CancelExecutionServiceAsync(base_url=self._base_url)
        self.runtime_cloud = RuntimeCloudServiceAsync(base_url=self._base_url)
        self.account_cloud_attachment_summary = (
            AccountCloudAttachmentSummaryServiceAsync(base_url=self._base_url)
        )
        self.account_cloud_attachment_properties_default = (
            AccountCloudAttachmentPropertiesDefaultServiceAsync(base_url=self._base_url)
        )
        self.runtime_properties = RuntimePropertiesServiceAsync(
            base_url=self._base_url
        )
        self.runtime_observability_settings = (
            RuntimeObservabilitySettingsServiceAsync(base_url=self._base_url)
        )
        self.cloud_attachment_secrets_configuration = (
            CloudAttachmentSecretsConfigurationServiceAsync(base_url=self._base_url)
        )
        self.shared_server_information = SharedServerInformationServiceAsync(
            base_url=self._base_url
        )
        self.shared_web_server = SharedWebServerServiceAsync(base_url=self._base_url)
        self.throughput_account = ThroughputAccountServiceAsync(base_url=self._base_url)
        self.throughput_account_group = ThroughputAccountGroupServiceAsync(
            base_url=self._base_url
        )
        self.tradacoms_connector_record = TradacomsConnectorRecordServiceAsync(
            base_url=self._base_url
        )
        self.trading_partner_component = TradingPartnerComponentServiceAsync(
            base_url=self._base_url
        )
        self.trading_partner_processing_group = (
            TradingPartnerProcessingGroupServiceAsync(base_url=self._base_url)
        )
        self.x12_connector_record = X12ConnectorRecordServiceAsync(
            base_url=self._base_url
        )
        self.atom_disk_space = AtomDiskSpaceServiceAsync(base_url=self._base_url)
        self.list_queues = ListQueuesServiceAsync(base_url=self._base_url)
        self.listener_status = ListenerStatusServiceAsync(base_url=self._base_url)
        self.organization_component = OrganizationComponentServiceAsync(
            base_url=self._base_url
        )
        self.shared_communication_channel_component = (
            SharedCommunicationChannelComponentServiceAsync(base_url=self._base_url)
        )
        self.account_group_integration_pack = AccountGroupIntegrationPackServiceAsync(
            base_url=self._base_url
        )
        self.publisher_integration_pack = PublisherIntegrationPackServiceAsync(
            base_url=self._base_url
        )
        self.release_integration_pack = ReleaseIntegrationPackServiceAsync(
            base_url=self._base_url
        )
        self.release_integration_pack_status = ReleaseIntegrationPackStatusServiceAsync(
            base_url=self._base_url
        )
        self.runtime_restart_request = RuntimeRestartRequestServiceAsync(
            base_url=self._base_url
        )
        self.refresh_secrets_manager = RefreshSecretsManagerServiceAsync(
            base_url=self._base_url
        )

        # Re-apply auth, URL, and timeout to the new async service instances
        # (super().__init__ applied these to sync services that were just replaced)
        self.set_access_token(access_token)
        self.set_basic_auth(username=username, password=password)
        self.set_base_url(self._base_url)
        self.set_timeout(timeout)




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .api_usage_count_expression import (
    ApiUsageCountExpression,
    ApiUsageCountExpressionGuard,
)
from .api_usage_count_simple_expression import ApiUsageCountSimpleExpression
from .api_usage_count_grouping_expression import ApiUsageCountGroupingExpression


@JsonMap({})
class ApiUsageCountQueryConfigQueryFilter(BaseModel):
    """ApiUsageCountQueryConfigQueryFilter

    :param expression: expression
    :type expression: ApiUsageCountExpression
    """

    def __init__(self, expression: ApiUsageCountExpression, **kwargs):
        """ApiUsageCountQueryConfigQueryFilter

        :param expression: expression
        :type expression: ApiUsageCountExpression
        """
        self.expression = ApiUsageCountExpressionGuard.return_one_of(expression)
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class ApiUsageCountQueryConfig(BaseModel):
    """ApiUsageCountQueryConfig

    :param query_filter: query_filter
    :type query_filter: ApiUsageCountQueryConfigQueryFilter
    """

    def __init__(self, query_filter: ApiUsageCountQueryConfigQueryFilter, **kwargs):
        """ApiUsageCountQueryConfig

        :param query_filter: query_filter
        :type query_filter: ApiUsageCountQueryConfigQueryFilter
        """
        self.query_filter = self._define_object(
            query_filter, ApiUsageCountQueryConfigQueryFilter
        )
        self._kwargs = kwargs




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .component_metadata import ComponentMetadata


@JsonMap(
    {
        "result": "Result",
        "id_": "id",
        "status_code": "statusCode",
        "error_message": "errorMessage",
    }
)
class ComponentMetadataBulkResponseResponse(BaseModel):
    """ComponentMetadataBulkResponseResponse

    :param result: result
    :type result: ComponentMetadata
    :param index: index, defaults to None
    :type index: int, optional
    :param id_: id_, defaults to None
    :type id_: str, optional
    :param status_code: status_code, defaults to None
    :type status_code: int, optional
    :param error_message: error_message, defaults to None
    :type error_message: str, optional
    """

    def __init__(
        self,
        result: ComponentMetadata,
        index: int = SENTINEL,
        id_: str = SENTINEL,
        status_code: int = SENTINEL,
        error_message: str = SENTINEL,
        **kwargs,
    ):
        """ComponentMetadataBulkResponseResponse

        :param result: result
        :type result: ComponentMetadata
        :param index: index, defaults to None
        :type index: int, optional
        :param id_: id_, defaults to None
        :type id_: str, optional
        :param status_code: status_code, defaults to None
        :type status_code: int, optional
        :param error_message: error_message, defaults to None
        :type error_message: str, optional
        """
        self.result = self._define_object(result, ComponentMetadata)
        if index is not SENTINEL:
            self.index = index
        if id_ is not SENTINEL:
            self.id_ = id_
        if status_code is not SENTINEL:
            self.status_code = status_code
        if error_message is not SENTINEL:
            self.error_message = error_message
        self._kwargs = kwargs


@JsonMap({})
class ComponentMetadataBulkResponse(BaseModel):
    """ComponentMetadataBulkResponse

    :param response: response, defaults to None
    :type response: List[ComponentMetadataBulkResponseResponse], optional
    """

    def __init__(
        self, response: List[ComponentMetadataBulkResponseResponse] = SENTINEL, **kwargs
    ):
        """ComponentMetadataBulkResponse

        :param response: response, defaults to None
        :type response: List[ComponentMetadataBulkResponseResponse], optional
        """
        if response is not SENTINEL:
            self.response = self._define_list(
                response, ComponentMetadataBulkResponseResponse
            )
        self._kwargs = kwargs




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .component_atom_attachment_simple_expression import (
    ComponentAtomAttachmentSimpleExpression,
)
from .component_atom_attachment_grouping_expression import (
    ComponentAtomAttachmentGroupingExpression,
)


class ComponentAtomAttachmentExpressionGuard(OneOfBaseModel):
    class_list = {
        "ComponentAtomAttachmentSimpleExpression": ComponentAtomAttachmentSimpleExpression,
        "ComponentAtomAttachmentGroupingExpression": ComponentAtomAttachmentGroupingExpression,
    }


ComponentAtomAttachmentExpression = Union[
    ComponentAtomAttachmentSimpleExpression, ComponentAtomAttachmentGroupingExpression
]




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .account_user_federation_simple_expression import (
    AccountUserFederationSimpleExpression,
)
from .account_user_federation_grouping_expression import (
    AccountUserFederationGroupingExpression,
)


class AccountUserFederationExpressionGuard(OneOfBaseModel):
    class_list = {
        "AccountUserFederationSimpleExpression": AccountUserFederationSimpleExpression,
        "AccountUserFederationGroupingExpression": AccountUserFederationGroupingExpression,
    }


AccountUserFederationExpression = Union[
    AccountUserFederationSimpleExpression, AccountUserFederationGroupingExpression
]




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .property import Property


@JsonMap({})
class Properties(BaseModel):
    """Properties

    :param property: property, defaults to None
    :type property: List[Property], optional
    """

    def __init__(self, property: List[Property] = SENTINEL, **kwargs):
        """Properties

        :param property: property, defaults to None
        :type property: List[Property], optional
        """
        if property is not SENTINEL:
            self.property = self._define_list(property, Property)
        self._kwargs = kwargs
